"""Template tag to print date
"""
from bson.objectid import ObjectId
from django import template
from dateutil import tz

register = template.Library()


@register.filter
def print_timestamp(object_id):
    """Print timestamp.

    Args:
        object_id:

    Returns:

    """
    return _get_datetime(object_id)


def _get_datetime(object_id, str_format="%m/%d/%Y %I:%M:%S %p"):
    """Return converted to UTC.

    Args:
        object_id:
        str_format:

    Returns:

    """
    object_id = ObjectId(object_id)
    from_zone = tz.tzutc()
    to_zone = tz.tzlocal()
    datetime_utc = object_id.generation_time
    datetime_utc = datetime_utc.replace(tzinfo=from_zone)
    datetime_local = datetime_utc.astimezone(to_zone)
    return datetime_local.strftime(str_format)



""" REST Views for XSLT Transformation
"""
from django.http import Http404
from django.utils.decorators import method_decorator
from rest_framework import status
from rest_framework.exceptions import ValidationError
from rest_framework.response import Response
from rest_framework.views import APIView

import core_main_app.components.xsl_transformation.api as xsl_api
from core_main_app.commons import exceptions
from core_main_app.rest.xsl_transformation.serializers import XslTransformationSerializer, TransformSerializer
from core_main_app.utils.decorators import api_staff_member_required


class XslTransformationList(APIView):
    """
        List, Create XSL document.
    """

    def get(self, request):
        """ Return http response with all xsl document.

            GET /rest/xslt

            Args:
                request:

            Returns:

            """
        try:
            # Get object
            xsl_object_list = xsl_api.get_all()
            # Serialize object
            return_value = XslTransformationSerializer(xsl_object_list, many=True)
            # Return response
            return Response(return_value.data)
        except Exception as api_exception:
            content = {'message': api_exception.message}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

    @method_decorator(api_staff_member_required())
    def post(self, request):
        """ Save an xslt.

            POST /rest/xslt
            {
                "name": "instance_name",
                "filename": "url",
                "content": "<content />",
            }

            Args:
                request:

            Returns:

            """
        try:
            # Build serializer
            xsl_serializer = XslTransformationSerializer(data=request.data)
            # Validate xsl
            xsl_serializer.is_valid(True)
            # save or update the object
            xsl_serializer.save()
            return Response(xsl_serializer.data, status=status.HTTP_201_CREATED)
        except ValidationError as validation_exception:
            content = {'message': validation_exception.detail}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except Exception as api_exception:
            content = {'message': api_exception.message}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


class XslTransformationDetail(APIView):
    """" Get, delete, patch an XSL document.
    """

    def get_object(self, pk):
        """ Retrieve an xsl document

        Args:
            pk:

        Returns:

        """
        try:
            return xsl_api.get_by_id(pk)
        except exceptions.DoesNotExist:
            raise Http404

    def get(self, request, pk):
        """ Get xslt by its id.

        GET /rest/xslt/pk

        Args:
            request:
            pk:

        Returns:

        """
        try:
            # Get object
            xsl_object = self.get_object(pk)
            # Serialize object
            return_value = XslTransformationSerializer(xsl_object)
            # Return response
            return Response(return_value.data)
        except Http404:
            content = {'message': 'Xsl not found.'}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {'message': api_exception.message}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

    @method_decorator(api_staff_member_required())
    def delete(self, request, pk):
        """ Delete xsl document by its id.

        DELETE /rest/xslt/pk

        Args:
            pk:

        Returns:

        """
        try:
            # Get object
            xsl_object = self.get_object(pk)
            # Remove the instance
            xsl_api.delete(xsl_object)
            # Return response
            return Response(status=status.HTTP_204_NO_CONTENT)
        except Http404:
            content = {'message': 'Xsl not found.'}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {'message': api_exception.message}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

    @method_decorator(api_staff_member_required())
    def patch(self, request, pk):
        """ Update xsl

        Args:
            request:
            pk:

        Returns:

        """
        try:
            # Get object
            xsl_object = self.get_object(pk)
            # Build serializer
            xsl_serializer = XslTransformationSerializer(instance=xsl_object,
                                                         data=request.data,
                                                         partial=True)
            # Validate xsl
            xsl_serializer.is_valid(True)
            # Save xsl
            xsl_serializer.save()
            return Response(xsl_serializer.data, status=status.HTTP_200_OK)
        except ValidationError as validation_exception:
            content = {'message': validation_exception.detail}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except Http404:
            content = {'message': 'Xsl not found.'}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {'message': api_exception.message}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


class XslTransformationTransform(APIView):
    """
        Transform an Xml using a specific Xslt
    """

    def post(self, request):
        """ Transform

        POST /rest/xslt/transform
        {
            "xml_content": "<xml />",
            "xslt_name": "name"
        }

        Args:
            request:

        Returns:

        """
        try:
            # Build serializer
            serializer = TransformSerializer(data=request.data)
            # Validate data
            serializer.is_valid(True)
            # transform
            return_value = xsl_api.xsl_transform(**serializer.validated_data)
            return Response(return_value, status=status.HTTP_200_OK)
        except ValidationError as validation_exception:
            content = {'message': validation_exception.detail}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except Exception as api_exception:
            content = {'message': api_exception.message}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)



""" Serializers for Django User
"""
from rest_framework import serializers
from django.contrib.auth.models import User


class UserSerializer(serializers.ModelSerializer):
    """ User serializer
    """
    class Meta:
        """ Meta
        """
        model = User
        fields = ('id', 'username')



""" Set of functions to define the rules for access control
"""

from django.contrib.auth.models import User

from core_main_app.components.workspace import api as workspace_api
from core_main_app.utils.access_control.exceptions import AccessControlError
from core_main_app.utils.raw_query.mongo_raw_query import add_access_criteria


def has_perm_administration(func, *args, **kwargs):
    """ Is the given user has administration rights.

        Args:
            func:
            *args:
            **kwargs:

        Returns:

        """
    try:
        if args[0].is_superuser:
            return func(*args, **kwargs)
    except Exception:
        pass
    raise AccessControlError("The user doesn't have enough rights to access this data.")


def can_read_or_write_data_workspace(func, workspace, user):
    """ Can user read or write in workspace.

    Args:
        func:
        workspace:
        user:

    Returns:

    """
    if user.is_superuser:
        return func(workspace, user)

    _check_can_read_or_write_workspace(workspace, user)
    return func(workspace, user)


def can_write_data_workspace(func, data, workspace, user):
    """ Can user write data in workspace.

    Args:
        func:
        data:
        workspace:
        user:

    Returns:

    """
    if user.is_superuser:
        return func(data, workspace, user)

    _check_can_write_data(data, user)
    if workspace is not None:
        _check_can_write_workspace(workspace, user)
    return func(data, workspace, user)


def _check_can_write_workspace(workspace, user):
    """ Check that user can write in the workspace.

    Args:
        workspace:
        user:

    Returns:

    """
    accessible_workspaces = workspace_api.get_all_workspaces_with_write_access_by_user(user)
    if workspace not in accessible_workspaces:
        raise AccessControlError("The user does not have the permission to write into this workspace.")


def _check_can_read_or_write_workspace(workspace, user):
    """ Check that user can read or write in the workspace.

    Args:
        workspace:
        user:

    Returns:

    """
    accessible_write_workspaces = workspace_api.get_all_workspaces_with_write_access_by_user(user)
    accessible_read_workspaces = workspace_api.get_all_workspaces_with_read_access_by_user(user)
    if workspace not in list(accessible_write_workspaces) + list(accessible_read_workspaces):
        raise AccessControlError("The user does not have the permission to write into this workspace.")


def can_read_list_data_id(func, list_data_id, user):
    """ Can read list of data.

    Args:
        func:
        list_data_id:
        user:

    Returns:

    """
    if user.is_superuser:
        return func(list_data_id, user)

    list_data = func(list_data_id, user)
    _check_can_read_data_list(list_data, user)

    return list_data


def can_read_data_id(func, data_id, user):
    """ Can read data.

    Args:
        func:
        data_id:
        user:

    Returns:

    """
    if user.is_superuser:
        return func(data_id, user)

    data = func(data_id, user)
    _check_can_read_data(data, user)
    return data


def can_write_data(func, data, user):
    """ Can write data.

    Args:
        func:
        data:
        user:

    Returns:

    """
    if user.is_superuser:
        return func(data, user)

    _check_can_write_data(data, user)
    return func(data, user)


def can_read_data(func, data, user):
    """ Can read data.

    Args:
        func:
        data:
        user:

    Returns:

    """
    if user.is_superuser:
        return func(data, user)

    _check_can_read_data(data, user)
    return func(data, user)


def can_read_data_query(func, query, user):
    """ Can read a data, given a query.

    Args:
        func:
        query:
        user:

    Returns:

    """
    if user.is_superuser:
        return func(query, user)

    # update the query
    query = _update_can_read_query(query, user)
    # get list of data
    data_list = func(query, user)
    # TODO: check if necessary because it is time consuming (checking that user has access to list of returned data)
    # check that user can access the list of data
    _check_can_read_data_list(data_list, user)
    return data_list


def can_read_user(func, user):
    """ Can read data, given a user.

    Args:
        func:
        user:

    Returns:

    """
    if user.is_superuser:
        return func(user)

    # get list of data
    data_list = func(user)
    # check that the user can access the list of data
    _check_can_read_data_list(data_list, user)
    # return list of data
    return data_list


def can_change_owner(func, data, new_user, user):
    """ Can user change data's owner.

    Args:
        func:
        data:
        new_user:
        user:

    Returns:

    """
    if user.is_superuser:
        return func(data, new_user, user)

    if data.user_id != str(user.id):
        raise AccessControlError("The user doesn't have enough rights to access this data.")

    return func(data, new_user, user)


def _check_can_write_data(data, user):
    """ Check that the user can write a data.

    Args:
        data:
        user:

    Returns:

    """
    if data.user_id != str(user.id):
        if hasattr(data, 'workspace') and data.workspace is not None:
            # get list of accessible workspaces
            accessible_workspaces = workspace_api.get_all_workspaces_with_write_access_by_user(user)
            # check that accessed data belongs to an accessible workspace
            if data.workspace not in accessible_workspaces:
                raise AccessControlError("The user doesn't have enough rights to access this data.")
        # workspace is not set
        else:
            raise AccessControlError("The user doesn't have enough rights to access this data.")


def _check_can_read_data(data, user):
    """ Check that the user can read a data.

    Args:
        data:
        user:

    Returns:

    """
    # workspace case
    if data.user_id != str(user.id):
        # workspace is set
        if hasattr(data, 'workspace') and data.workspace is not None:
            # get list of accessible workspaces
            accessible_workspaces = workspace_api.get_all_workspaces_with_read_access_by_user(user)
            # check that accessed data belongs to an accessible workspace
            if data.workspace not in accessible_workspaces:
                raise AccessControlError("The user doesn't have enough rights to access this data.")
        # workspace is not set
        else:
            raise AccessControlError("The user doesn't have enough rights to access this data.")


def _check_can_read_data_list(data_list, user):
    """ Check that the user can read each data of the list.

    Args:
        data_list:
        user:

    Returns:

    """
    if len(data_list) > 0:
        # get list of accessible workspaces
        accessible_workspaces = workspace_api.get_all_workspaces_with_read_access_by_user(user)
        # check access is correct
        for data in data_list:
            # user is data owner
            if data.user_id == str(user.id):
                continue
            # user is not owner or data not in accessible workspace
            if data.workspace is None or data.workspace not in accessible_workspaces:
                raise AccessControlError("The user doesn't have enough rights to access this data.")


def _update_can_read_query(query, user):
    """ Update query with access control parameters.

    Args:
        query:
        user:

    Returns:

    """

    # workspace case
    # list accessible workspaces
    accessible_workspaces = [workspace.id for workspace in
                             workspace_api.get_all_workspaces_with_read_access_by_user(user)]
    # update query with workspace criteria
    query = add_access_criteria(query, accessible_workspaces, user)
    return query



"""
    Admin views
"""

from django.contrib.admin.views.decorators import staff_member_required
from django.core.urlresolvers import reverse
from django.http.response import HttpResponseRedirect
from django.template import loader
from django.utils.html import escape as html_escape
from django.views.generic import View

from core_main_app.commons import exceptions
from core_main_app.components.template.models import Template
from core_main_app.components.template_version_manager import api as template_version_manager_api
from core_main_app.components.template_version_manager.models import TemplateVersionManager
from core_main_app.components.version_manager import api as version_manager_api
from core_main_app.components.xsl_transformation import api as xslt_transformation_api
from core_main_app.components.xsl_transformation.models import XslTransformation
from core_main_app.utils.rendering import admin_render
from core_main_app.utils.xml import get_imports_and_includes
from core_main_app.views.admin.ajax import EditXSLTView
from core_main_app.views.admin.forms import UploadTemplateForm, UploadVersionForm, UploadXSLTForm
from core_main_app.views.common.ajax import EditTemplateVersionManagerView, DeleteObjectModalView
from core_main_app.views.common.views import read_xsd_file
from core_main_app.views.user.views import get_context_manage_template_versions


@staff_member_required
def admin_home(request):
    """Admin home view.

    Args:
        request:

    Returns:

    """
    return admin_render(request,
                        'core_main_app/admin/dashboard.html')


@staff_member_required
def manage_templates(request):
    """View that allows template management.

    Args:
        request:

    Returns:

    """
    # get all current templates
    templates = template_version_manager_api.get_global_version_managers()

    context = {
        'object_name': 'Template',
        'available': [template for template in templates if not template.is_disabled],
        'disabled': [template for template in templates if template.is_disabled]
    }

    assets = {
                "js": [
                    {
                        "path": 'core_main_app/common/js/templates/list/restore.js',
                        "is_raw": False
                    },
                    {
                        "path": 'core_main_app/common/js/templates/list/modals/disable.js',
                        "is_raw": False
                    },
                    EditTemplateVersionManagerView.get_modal_js_path()
                ]
            }

    modals = [
                "core_main_app/admin/templates/list/modals/disable.html",
                EditTemplateVersionManagerView.get_modal_html_path()
            ]

    return admin_render(request,
                        'core_main_app/admin/templates/list.html',
                        assets=assets,
                        context=context,
                        modals=modals)


@staff_member_required
def manage_template_versions(request, version_manager_id):
    """View that allows template versions management.

    Args:
        request:
        version_manager_id:

    Returns:

    """

    # get the version manager
    version_manager = None
    try:
        version_manager = version_manager_api.get(version_manager_id)
    except:
        # TODO: catch good exception, redirect to error page
        pass

    context = get_context_manage_template_versions(version_manager)

    assets = {
                "js": [
                    {
                        "path": 'core_main_app/common/js/templates/versions/set_current.js',
                        "is_raw": False
                    },
                    {
                        "path": 'core_main_app/common/js/templates/versions/restore.js',
                        "is_raw": False
                    },
                    {
                        "path": 'core_main_app/common/js/templates/versions/modals/disable.js',
                        "is_raw": False
                    },
                    {
                        "path": 'core_main_app/common/js/backtoprevious.js',
                        "is_raw": True
                    }
                ]
            }

    modals = ["core_main_app/admin/templates/versions/modals/disable.html"]

    return admin_render(request,
                        'core_main_app/admin/templates/versions.html',
                        assets=assets,
                        modals=modals,
                        context=context)


@staff_member_required
def upload_template(request):
    """Upload template.

    Args:
        request:

    Returns:

    """
    assets = {
        "js": [
            {
                "path": 'core_main_app/admin/js/templates/upload/dependency_resolver.js',
                "is_raw": True
            },
            {
                "path": 'core_main_app/admin/js/templates/upload/dependencies.js',
                "is_raw": False
            },
            {
                "path": 'core_main_app/common/js/backtoprevious.js',
                "is_raw": True
            }
        ]
    }

    context = {
        'object_name': 'Template',
        'url': reverse("admin:core_main_app_upload_template"),
        'redirect_url': reverse("admin:core_main_app_templates")
    }

    # method is POST
    if request.method == 'POST':
        form = UploadTemplateForm(request.POST,  request.FILES)
        context['upload_form'] = form

        if form.is_valid():
            return _save_template(request, assets, context)
        else:
            # Display error from the form
            return _upload_template_response(request, assets, context)
    # method is GET
    else:
        # render the form to upload a template
        context['upload_form'] = UploadTemplateForm()
        return _upload_template_response(request, assets, context)


@staff_member_required
def upload_template_version(request, version_manager_id):
    """Upload template version.

    Args:
        request:
        version_manager_id:

    Returns:

    """
    assets = {
        "js": [
            {
                "path": 'core_main_app/admin/js/templates/upload/dependency_resolver.js',
                "is_raw": True
            },
            {
                "path": 'core_main_app/admin/js/templates/upload/dependencies.js',
                "is_raw": False
            }
        ]
    }

    template_version_manager = version_manager_api.get(version_manager_id)
    context = {
        'object_name': "Template",
        'version_manager': template_version_manager,
        'url': reverse("admin:core_main_app_upload_template_version",
                       kwargs={'version_manager_id': template_version_manager.id}),
        'redirect_url': reverse("admin:core_main_app_manage_template_versions",
                                kwargs={'version_manager_id': template_version_manager.id})
    }

    # method is POST
    if request.method == 'POST':
        form = UploadVersionForm(request.POST,  request.FILES)
        context['upload_form'] = form

        if form.is_valid():
            return _save_template_version(request, assets, context, template_version_manager)
        else:
            # Display errors from the form
            return _upload_template_response(request, assets, context)
    # method is GET
    else:
        # render the form to upload a template
        context['upload_form'] = UploadVersionForm()
        return _upload_template_response(request, assets, context)


def _save_template(request, assets, context):
    """Save a template.

    Args:
        request:
        assets:
        context:

    Returns:

    """
    # get the schema name
    name = request.POST['name']
    # get the file from the form
    xsd_file = request.FILES['upload_file']
    # read the content of the file
    xsd_data = read_xsd_file(xsd_file)

    try:
        template = Template(filename=xsd_file.name, content=xsd_data)
        template_version_manager = TemplateVersionManager(title=name)
        template_version_manager_api.insert(template_version_manager, template)
        return HttpResponseRedirect(reverse("admin:core_main_app_templates"))
    except exceptions.XSDError, xsd_error:
        return handle_xsd_errors(request, assets, context, xsd_error, xsd_data, xsd_file.name)
    except exceptions.NotUniqueError:
        context['errors'] = html_escape("A template with the same name already exists. Please choose another name.")
        return _upload_template_response(request, assets, context)
    except Exception, e:
        context['errors'] = html_escape(e.message)
        return _upload_template_response(request, assets, context)


def _save_template_version(request, assets, context, template_version_manager):
    """Save a template version.

    Args:
        request:
        assets:
        context:
        template_version_manager:

    Returns:

    """
    # get the file from the form
    xsd_file = request.FILES['xsd_file']
    # read the content of the file
    xsd_data = read_xsd_file(xsd_file)

    try:
        template = Template(filename=xsd_file.name, content=xsd_data)
        template_version_manager_api.insert(template_version_manager, template)
        return HttpResponseRedirect(reverse("admin:core_main_app_manage_template_versions",
                                            kwargs={'version_manager_id': str(template_version_manager.id)}))
    except exceptions.XSDError, xsd_error:
        return handle_xsd_errors(request, assets, context, xsd_error, xsd_data, xsd_file.name)
    except Exception, e:
        context['errors'] = html_escape(e.message)
        return _upload_template_response(request, assets, context)


def _upload_template_response(request, assets, context):
    """Render template upload response.

    Args:
        request:
        context:

    Returns:

    """
    return admin_render(request,
                        'core_main_app/admin/templates/upload.html',
                        assets=assets,
                        context=context)


class XSLTView(View):
    """XSLT view.
    """

    @staticmethod
    def get(request, *args, **kwargs):
        modals = [
            EditXSLTView.get_modal_html_path(),
            DeleteObjectModalView.get_modal_html_path()
        ]

        assets = {
            "js": [
                EditXSLTView.get_modal_js_path(),
                DeleteObjectModalView.get_modal_js_path()
            ],
        }

        context = {
            'object_name': 'XSLT',
            "xslt": xslt_transformation_api.get_all(),
            "update_url": reverse('admin:core_main_app_upload_xslt')
        }

        return admin_render(request, "core_main_app/admin/xslt/list.html", modals=modals,
                            assets=assets, context=context)


class UploadXSLTView(View):
    """Upload XSLT view.
    """
    form_class = UploadXSLTForm
    template_name = 'core_main_app/admin/xslt/upload.html'
    object_name = 'XSLT'

    def __init__(self, **kwargs):
        super(UploadXSLTView, self).__init__(**kwargs)
        self.context = {}
        self.context.update({'object_name': self.object_name})

    def get(self, request, *args, **kwargs):
        self.context.update({'upload_form': self.form_class()})
        return admin_render(request, self.template_name, context=self.context)

    def post(self, request, *args, **kwargs):
        form = self.form_class(request.POST, request.FILES)
        self.context.update({'upload_form': form})

        if form.is_valid():
            return self._save_xslt(request)
        else:
            # Display error from the form
            return admin_render(request, self.template_name, context=self.context)

    def _save_xslt(self, request):
        """Saves an XSLT.

        Args:
            request: Request.

        """
        try:
            # get the XSLT name
            name = request.POST['name']
            # get the file from the form
            xsd_file = request.FILES['upload_file']
            # read the content of the file
            xsd_data = read_xsd_file(xsd_file)
            xslt = XslTransformation(name=name, filename=xsd_file.name, content=xsd_data)
            xslt_transformation_api.upsert(xslt)

            return HttpResponseRedirect(reverse("admin:core_main_app_xslt"))
        except exceptions.NotUniqueError:
            self.context.update({'errors': html_escape("This name already exists.")})
            return admin_render(request, 'core_main_app/admin/xslt/upload.html', context=self.context)
        except Exception, e:
            self.context.update({'errors': html_escape(e.message)})
            return admin_render(request, 'core_main_app/admin/xslt/upload.html', context=self.context)


def handle_xsd_errors(request, assets, context, xsd_error, xsd_content, filename):
    """Handle XSD errors. Builds dependency resolver if needed.

    Args:
        request:
        assets:
        context:
        xsd_error:
        xsd_content:
        filename:

    Returns:

    """
    imports, includes = get_imports_and_includes(xsd_content)
    # a problem with includes/imports has been detected
    if len(includes) > 0 or len(imports) > 0:
        # build dependency resolver
        context['dependency_resolver'] = get_dependency_resolver_html(imports, includes, xsd_content, filename)
        return _upload_template_response(request, assets, context)
    else:
        context['errors'] = html_escape(xsd_error.message)
        return _upload_template_response(request, assets, context)


def get_dependency_resolver_html(imports, includes, xsd_data, filename):
    """Return HTML for dependency resolver form.

    Args:
        imports:
        includes:
        xsd_data:
        filename:

    Returns:

    """
    # build the list of dependencies
    current_templates = template_version_manager_api.get_global_version_managers(_cls=False)
    list_dependencies_template = loader.get_template('core_main_app/admin/list_dependencies.html')
    context = {
        'templates': [template for template in current_templates if not template.is_disabled],
    }
    list_dependencies_html = list_dependencies_template.render(context)

    # build the dependency resolver form
    dependency_resolver_template = loader.get_template('core_main_app/admin/dependency_resolver.html')
    context = {
        'imports': imports,
        'includes': includes,
        'xsd_content': html_escape(xsd_data),
        'filename': filename,
        'dependencies': list_dependencies_html,
    }
    return dependency_resolver_template.render(context)



"""Serializers for Xsl transformation
"""
from rest_framework.serializers import CharField
from rest_framework_mongoengine.serializers import DocumentSerializer

import core_main_app.components.xsl_transformation.api as xsl_api
from core_main_app.commons.serializers import BasicSerializer
from core_main_app.components.xsl_transformation.models import XslTransformation


class XslTransformationSerializer(DocumentSerializer):
    """
        XslTransformation serializer
    """
    class Meta:
        model = XslTransformation
        fields = "__all__"

    def create(self, validated_data):
        return xsl_api.upsert(XslTransformation(**validated_data))

    def update(self, instance, validated_data):
        instance.name = validated_data.get('name', instance.name)
        instance.content = validated_data.get('content', instance.content)
        instance.filename = validated_data.get('filename', instance.filename)
        return xsl_api.upsert(instance)


class TransformSerializer(BasicSerializer):
    xml_content = CharField(required=True)
    xslt_name = CharField(required=True)



"""Unit tests for data rest api
"""
from django.test import SimpleTestCase
from mock.mock import patch
from rest_framework import status

from core_main_app.commons.exceptions import DoesNotExist
from core_main_app.components.data.models import Data
from core_main_app.rest.data import views as data_rest_views
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock


class TestDataList(SimpleTestCase):
    def setUp(self):
        super(TestDataList, self).setUp()
        self.data = {'template': '1',
                     'user_id': '1',
                     'xml_content': 'test',
                     'title': 'test'}

    @patch.object(Data, 'get_all')
    def test_get_all_returns_http_200(self, mock_get_all):
        # Arrange
        mock_user = create_mock_user('1')
        mock_get_all.return_value = []

        # Mock
        response = RequestMock.do_request_get(data_rest_views.DataList.as_view(),
                                              mock_user)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_post_returns_http_400_if_invalid_data(self):
        # Arrange
        mock_user = create_mock_user('1')

        # Mock
        response = RequestMock.do_request_post(data_rest_views.DataList.as_view(),
                                               mock_user,
                                               data=self.data)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)


class TestDataDetail(SimpleTestCase):
    def setUp(self):
        super(TestDataDetail, self).setUp()

    @patch.object(Data, 'get_by_id')
    def test_get_returns_http_404_when_data_not_found(self, mock_get_by_id):
        # Arrange
        mock_user = create_mock_user('1')
        mock_get_by_id.side_effect = DoesNotExist("error")

        # Mock
        response = RequestMock.do_request_get(data_rest_views.DataDetail.as_view(),
                                              mock_user,
                                              param={'pk': '1'})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    @patch.object(Data, 'get_by_id')
    def test_delete_returns_http_404_when_data_not_found(self, mock_get_by_id):
        # Arrange
        mock_user = create_mock_user('1')
        mock_get_by_id.side_effect = DoesNotExist("error")

        # Mock
        response = RequestMock.do_request_delete(data_rest_views.DataDetail.as_view(),
                                                 mock_user,
                                                 param={'pk': '1'})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    @patch.object(Data, 'get_by_id')
    def test_patch_returns_http_404_when_data_not_found(self, mock_get_by_id):
        # Arrange
        mock_user = create_mock_user('1')
        mock_get_by_id.side_effect = DoesNotExist("error")

        # Mock
        response = RequestMock.do_request_patch(data_rest_views.DataDetail.as_view(),
                                                mock_user,
                                                param={'pk': '1'})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    @patch.object(Data, 'get_by_id')
    @patch('core_main_app.components.data.api.upsert')
    def test_patch_returns_http_400_when_no_update_data_provided(self, mock_get_by_id, mock_upsert):
        # Arrange
        mock_user = create_mock_user('1')
        mock_data = Data(user_id='1')
        mock_get_by_id.return_value = mock_data
        mock_upsert.return_value = mock_data

        # Mock
        response = RequestMock.do_request_patch(data_rest_views.DataDetail.as_view(),
                                                mock_user,
                                                param={'pk': '1'})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)


class TestDataDownload(SimpleTestCase):
    def setUp(self):
        super(TestDataDownload, self).setUp()

    @patch.object(Data, 'get_by_id')
    def test_get_returns_http_404_when_data_not_found(self, mock_get_by_id):
        # Arrange
        mock_user = create_mock_user('1')
        mock_get_by_id.side_effect = DoesNotExist("error")

        # Mock
        response = RequestMock.do_request_get(data_rest_views.DataDownload.as_view(),
                                              mock_user,
                                              param={'pk': '1'})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)



""" REST views for the data API
"""
import json

from django.http import Http404
from rest_framework import status
from rest_framework.decorators import api_view
from rest_framework.exceptions import ValidationError
from rest_framework.response import Response
from rest_framework.views import APIView

from core_main_app.commons import exceptions
from core_main_app.components.data import api as data_api
from core_main_app.rest.data.abstract_views import AbstractExecuteLocalQueryView
from core_main_app.rest.data.serializers import DataSerializer, DataWithTemplateInfoSerializer
from core_main_app.utils.boolean import to_bool
from core_main_app.utils.databases.pymongo_database import get_full_text_query
from core_main_app.utils.file import get_file_http_response
from core_main_app.utils.pagination.rest_framework_paginator.pagination import StandardResultsSetPagination


# FIXME: permissions

class DataList(APIView):
    """ List all user data, or create a new one.
    """
    def get(self, request):
        """ Get all user data.

        Query Params:
            template: template id
            title: title

        Args:
            request:

        Returns:

        """
        try:
            # Get object
            data_object_list = data_api.get_all_by_user(request.user)

            # Apply filters
            template = self.request.query_params.get('template', None)
            if template is not None:
                data_object_list = data_object_list.filter(template=template)

            title = self.request.query_params.get('title', None)
            if title is not None:
                data_object_list = data_object_list.filter(title=title)

            # Serialize object
            data_serializer = DataSerializer(data_object_list, many=True)

            # Return response
            return Response(data_serializer.data, status=status.HTTP_200_OK)
        except Exception as api_exception:
            content = {'message': api_exception.message}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

    def post(self, request):
        """ Create data

        Args:
            request:

        Returns:

        """
        try:
            # Build serializer
            data_serializer = DataSerializer(data=request.data)

            # Validate data
            data_serializer.is_valid(True)
            # Save data
            data_serializer.save(user=request.user)

            # Return the serialized data
            return Response(data_serializer.data, status=status.HTTP_201_CREATED)
        except ValidationError as validation_exception:
            content = {'message': validation_exception.detail}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except exceptions.DoesNotExist:
            content = {'message': 'Template not found.'}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except Exception as api_exception:
            content = {'message': api_exception.message}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


class DataDetail(APIView):
    """
    Retrieve, update or delete a data.
    """
    def get_object(self, request, pk):
        """ Get data from db

        Args:
            request:
            pk:

        Returns:

        """
        try:
            return data_api.get_by_id(pk, request.user)
        except exceptions.DoesNotExist:
            raise Http404

    def get(self, request, pk):
        """ Retrieve data

        Args:
            request:
            pk:

        Returns:

        """
        try:
            # Get object
            data_object = self.get_object(request, pk)

            # Serialize object
            serializer = DataSerializer(data_object)

            # Return response
            return Response(serializer.data)
        except Http404:
            content = {'message': 'Data not found.'}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {'message': api_exception.message}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

    def delete(self, request, pk):
        """ Delete a data

        Args:
            request:
            pk:

        Returns:

        """
        try:
            # Get object
            data_object = self.get_object(request, pk)

            # delete object
            data_api.delete(data_object, request.user)

            # Return response
            return Response(status=status.HTTP_204_NO_CONTENT)
        except Http404:
            content = {'message': 'Data not found.'}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {'message': api_exception.message}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

    def patch(self, request, pk):
        """ Update data

        Args:
            request:
            pk:

        Returns:

        """
        try:
            # Get object
            data_object = self.get_object(request, pk)

            # Build serializer
            data_serializer = DataSerializer(instance=data_object,
                                             data=request.data,
                                             partial=True)

            # Validate data
            data_serializer.is_valid(True)
            # Save data
            data_serializer.save(user=request.user)

            return Response(data_serializer.data, status=status.HTTP_200_OK)
        except ValidationError as validation_exception:
            content = {'message': validation_exception.detail}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except Http404:
            content = {'message': 'Data not found.'}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {'message': api_exception.message}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


class DataDownload(APIView):
    """
        Download XML file in data.
    """
    def get_object(self, request, pk):
        """ Get data from db

        Args:
            request:
            pk:

        Returns:

        """
        try:
            return data_api.get_by_id(pk, request.user)
        except exceptions.DoesNotExist:
            raise Http404

    def get(self, request, pk):
        """ Download data

        Args:
            request:
            pk:

        Returns:

        """
        try:
            # Get object
            data_object = self.get_object(request, pk)

            return get_file_http_response(data_object.xml_content, data_object.title, 'text/xml', 'xml')
        except Http404:
            content = {'message': 'Data not found.'}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {'message': api_exception.message}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


# FIXME: Should use in the future an serializer with dynamic fields (init depth with parameter for example)
# Should avoid here a duplication code with get_by_id
@api_view(['GET'])
def get_by_id_with_template_info(request):
    """ Get data by its id.

        /rest/data/get-full?id=588a73b47179c722f6fdaf43

        Args:
            request:

        Returns:

        """
    try:
        # Get parameters
        data_id = request.query_params.get('id', None)

        # Check parameters
        if data_id is None:
            content = {'message': 'Expected parameters not provided.'}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)

        # Get object
        data_object = data_api.get_by_id(data_id, request.user)

        # Serialize object
        return_value = DataWithTemplateInfoSerializer(data_object)

        # Return response
        return Response(return_value.data, status=status.HTTP_200_OK)
    except exceptions.DoesNotExist as e:
        content = {'message': 'No data found with the given id.'}
        return Response(content, status=status.HTTP_404_NOT_FOUND)
    except Exception as api_exception:
        content = {'message': api_exception.message}
        return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


class ExecuteLocalQueryView(AbstractExecuteLocalQueryView):
    def post(self, request):
        """ Execute a query.

        /rest/data/query/
        /rest/data/query/?page=2

        Example Data:
            {"query": "{\"root.element.value\": 2}"}
            {"query": "{\"root.element.value\": 2}", "all": "true"}
            {"query": "{\"root.element.value\": 2}", "templates": "[{\"id\":\"<template_id>\"}]"}
            {"query": "{}", "templates": "[{\"id\":\"<template_id>\"}]"}

        Args:
            request:

        Returns:

        """
        return super(ExecuteLocalQueryView, self).post(request)

    def build_response(self, data_list):
        """ Build the response.

        Args:
            data_list: List of data.

        Returns:
            The response.

        """
        if 'all' in self.request.data and to_bool(self.request.data['all']):
            # Serialize data list
            data_serializer = DataSerializer(data_list, many=True)
            # Return response
            return Response(data_serializer.data)
        else:
            # Get paginator
            paginator = StandardResultsSetPagination()

            # Get requested page from list of results
            page = paginator.paginate_queryset(data_list, self.request)

            # Serialize page
            data_serializer = DataSerializer(page, many=True)

            # Return paginated response
            return paginator.get_paginated_response(data_serializer.data)


class ExecuteLocalKeywordQueryView(ExecuteLocalQueryView):
    def build_query(self, query, templates):
        """ Build the raw query. Prepare the query for a keyword search.
        Args:
            query:
            templates:

        Returns:
            The raw query.

        """
        # build query builder
        query = json.dumps(get_full_text_query(query))
        return super(ExecuteLocalKeywordQueryView, self).build_query(str(query), templates)



"""Serializers used throughout the Rest API
"""
from rest_framework.fields import CharField
from rest_framework_mongoengine.serializers import DocumentSerializer

from core_main_app.components.template.models import Template


class TemplateSerializer(DocumentSerializer):
    """
        Template serializer
    """
    dependencies_dict = CharField(write_only=True, required=False)

    class Meta:
        model = Template
        fields = ['id',
                  'filename',
                  'content',
                  'hash',
                  'dependencies',
                  'dependencies_dict']

        read_only_fields = ['id',
                            'hash',
                            '_display_name', ]



"""
    Lock API
"""
import datetime

from core_main_app.commons.exceptions import LockError
from core_main_app.components.lock.models import Lock
from core_main_app.settings import LOCK_OBJECT_TTL


def is_object_locked(object, user):
    """ Check if the object is locked.

    Args:
        object:
    Returns:
    """
    try:
        lock = Lock.acquire()
        _check_object_locked(object, user, lock)
        return False
    except LockError, ler:
        return True
    finally:
        Lock.release()


def set_lock_object(object, user):
    """ Set lock on object.

    Args:
        object:
        user:
    Returns:
    """
    try:
        lock = Lock.acquire()
        if not _check_object_locked(object, user, lock):
            lock.set_lock(object, user)
    except LockError, ler:
        raise ler
    finally:
        Lock.release()


def remove_lock_on_object(object, user):
    """ Remove lock on object.

    Args:
        object:
        user:
    Returns:
    """
    try:
        lock = Lock.acquire()
        database_lock_object = lock.get_object_locked(object)
        # Only the user who created the lock can remove it
        if database_lock_object.user_id == str(user.id):
            lock.remove_lock(database_lock_object)
    except:
        pass
    finally:
        Lock.release()


def _check_object_locked(object, user, lock):
    """ Check all conditions of a lock object to define if it is locked or not.
        If there is no lock on object, return false.
        If there is a lock but owned by the user, return true.
        If there is a lock no owned by the user, raise LockError exception.

    Args:
        object:
        user:
        lock:

    Returns:
    """
    try:
        database_lock_object = lock.get_object_locked(object)
    except:
        return False

    # Check if lock has expired
    date = database_lock_object.lock_date
    if (datetime.datetime.now() - date).total_seconds() > LOCK_OBJECT_TTL:
        lock.remove_lock(database_lock_object)

    # If the user who requested the object is the same as the one who locked the object
    # then it is not locked for him
    if database_lock_object.user_id == str(user.id):
        return True

    raise LockError('The object is used by another user and is locked.')



"""
    Utils for groups
"""


def remove_list_object_from_list(list_object, list_object_to_be_removed):
    """ Remove list of objects from another list of objects.

    Args:
            list_object:
            list_object_to_be_removed:
    Returns:
    """

    for group in list_object_to_be_removed:
        if group in list_object:
            list_object.remove(group)




""" Fixtures files for template version manager
"""
from core_main_app.components.template.models import Template
from core_main_app.components.template_version_manager.models import TemplateVersionManager
from core_main_app.utils.integration_tests.fixture_interface import FixtureInterface


class TemplateVersionManagerFixtures(FixtureInterface):
    """ Template Version Manager fixtures
    """
    template_1_1 = None
    template_1_2 = None
    template_1_3 = None
    template_2_1 = None
    template_vm_1 = None
    template_vm_2 = None
    template_vm_collection = None

    def insert_data(self):
        """ Insert a set of Templates and Template Version Managers.

        Returns:

        """
        # Make a connexion with a mock database
        self.template_1_1 = Template(filename="template1_1.xsd",
                                     content="content1_1",
                                     hash="hash1_1").save()
        self.template_1_2 = Template(filename="template1_2.xsd",
                                     content="content1_2",
                                     hash="hash1_2").save()
        self.template_1_3 = Template(filename="template1_3.xsd",
                                     content="content1_3",
                                     hash="hash1_3").save()
        self.template_2_1 = Template(filename="template2_1.xsd",
                                     content="content2_1",
                                     hash="hash2_1").save()

        self.template_vm_1 = TemplateVersionManager(title="template 1",
                                                    user=None,
                                                    versions=[str(self.template_1_1.id),
                                                              str(self.template_1_2.id),
                                                              str(self.template_1_3.id)],
                                                    current=str(self.template_1_3.id),
                                                    is_disabled=False,
                                                    disabled_versions=[str(self.template_1_2.id)]).save()

        self.template_vm_2 = TemplateVersionManager(title="template 2",
                                                    user="1",
                                                    versions=[str(self.template_2_1.id)],
                                                    current=str(self.template_2_1.id),
                                                    is_disabled=False,
                                                    disabled_versions=[]).save()

        self.template_vm_collection = [self.template_vm_1, self.template_vm_2]



"""Integration tests for xsl transformation rest api
"""
from bson import ObjectId
from rest_framework import status

import core_main_app.rest.xsl_transformation.views as xsl_views
from core_main_app.utils.integration_tests.integration_base_test_case import MongoIntegrationBaseTestCase
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock
from tests.rest.xsl_transformation.fixtures.fixtures import XslTransformationFixtures

fixture_data = XslTransformationFixtures()


class TestGetAllXslTransformationList(MongoIntegrationBaseTestCase):
    fixture = fixture_data

    def setUp(self):
        super(TestGetAllXslTransformationList, self).setUp()

    def test_get_all_returns_status_200_with_no_permission_needed(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_get(xsl_views.XslTransformationList.as_view(),
                                              user)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestPostXslTransformationList(MongoIntegrationBaseTestCase):
    fixture = fixture_data

    def setUp(self):
        super(TestPostXslTransformationList, self).setUp()
        self.data = None

    def test_post_returns_status_403_if_user_is_unauthorized(self):
        # Arrange
        user = create_mock_user('0')

        # Act
        response = RequestMock.do_request_post(xsl_views.XslTransformationList.as_view(),
                                               user,
                                               self.data)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_post_returns_status_400_if_data_are_not_valid_with_admin_user(self):
        # Arrange
        user = create_mock_user('0', True, True)

        # Act
        response = RequestMock.do_request_post(xsl_views.XslTransformationList.as_view(),
                                               user,
                                               self.data)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_post_returns_status_201_if_data_are_valid_with_admin_user(self):
        # Arrange
        user = create_mock_user('0', True, True)
        self.data = {
            'name': 'name',
            'filename': 'filename.xsd',
            'content': '<?xml version=\"1.0\" encoding=\"UTF-8\"?>'
                       '<xsl:stylesheet xmlns:xsl=\"http://www.w3.org/1999/XSL/Transform\" version=\"1.0\">'
                       '<xsl:template></xsl:template></xsl:stylesheet>'
        }

        # Act
        response = RequestMock.do_request_post(xsl_views.XslTransformationList.as_view(),
                                               user,
                                               self.data)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)


class TestGetXslTransformationDetail(MongoIntegrationBaseTestCase):
    fixture = fixture_data

    def setUp(self):
        super(TestGetXslTransformationDetail, self).setUp()
        self.data = None

    def test_get_returns_object_when_found(self):
        # Arrange
        user = create_mock_user('0')
        self.param = {
            'pk': self.fixture.data_1.id
        }

        # Act
        response = RequestMock.do_request_get(xsl_views.XslTransformationDetail.as_view(),
                                              user,
                                              self.data,
                                              self.param)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_get_raise_404_when_not_found(self):
        # Arrange
        user = create_mock_user('0')
        self.param = {
            'pk': str(ObjectId())
        }

        # Act
        response = RequestMock.do_request_get(xsl_views.XslTransformationDetail.as_view(),
                                              user,
                                              self.data,
                                              self.param)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_get_raise_500_sever_error_when_general_error_occured(self):
        # Arrange
        user = create_mock_user('0')
        self.param = {
            'pk': '0'
        }

        # Act
        response = RequestMock.do_request_get(xsl_views.XslTransformationDetail.as_view(),
                                              user,
                                              self.data,
                                              self.param)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_500_INTERNAL_SERVER_ERROR)


class TestDeleteXslTransformationDetail(MongoIntegrationBaseTestCase):
    fixture = fixture_data

    def setUp(self):
        super(TestDeleteXslTransformationDetail, self).setUp()
        self.data = None

    def test_delete_raise_403_if_user_is_unauthorized(self):
        # Arrange
        user = create_mock_user('0')
        self.param = {
            'pk': str(ObjectId())
        }

        # Act
        response = RequestMock.do_request_delete(xsl_views.XslTransformationDetail.as_view(),
                                                 user,
                                                 self.data,
                                                 self.param)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_delete_raise_404_when_not_found(self):
        # Arrange
        user = create_mock_user('0', True, True)
        self.param = {
            'pk': str(ObjectId())
        }

        # Act
        response = RequestMock.do_request_delete(xsl_views.XslTransformationDetail.as_view(),
                                                 user,
                                                 self.data,
                                                 self.param)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_post_raise_500_sever_error_when_general_error_occured(self):
        # Arrange
        user = create_mock_user('0', True, True)
        self.param = {
            'pk': '0'
        }

        # Act
        response = RequestMock.do_request_delete(xsl_views.XslTransformationDetail.as_view(),
                                                 user,
                                                 self.data,
                                                 self.param)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_500_INTERNAL_SERVER_ERROR)

    def test_post_return_204_if_document_is_deleted_whit_success(self):
        # Arrange
        user = create_mock_user('0', True, True)
        self.param = {
            'pk': self.fixture.data_1.id
        }

        # Act
        response = RequestMock.do_request_delete(xsl_views.XslTransformationDetail.as_view(),
                                                 user,
                                                 self.data,
                                                 self.param)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)


class TestPatchXslTransformationDetail(MongoIntegrationBaseTestCase):
    fixture = fixture_data

    def setUp(self):
        super(TestPatchXslTransformationDetail, self).setUp()
        self.data = None

    def test_patch_raise_403_if_user_is_authorized(self):
        # Arrange
        user = create_mock_user('0')
        self.param = {
            'pk': str(ObjectId())
        }

        # Act
        response = RequestMock.do_request_patch(xsl_views.XslTransformationDetail.as_view(),
                                                user,
                                                self.data,
                                                self.param)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_patch_raise_404_when_not_found(self):
        # Arrange
        user = create_mock_user('0', True, True)
        self.param = {
            'pk': str(ObjectId())
        }

        # Act
        response = RequestMock.do_request_patch(xsl_views.XslTransformationDetail.as_view(),
                                                user,
                                                self.data,
                                                self.param)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_patch_raise_500_sever_error_when_general_error_occured(self):
        # Arrange
        user = create_mock_user('0', True, True)
        self.param = {
            'pk': '0'
        }

        # Act
        response = RequestMock.do_request_patch(xsl_views.XslTransformationDetail.as_view(),
                                                user,
                                                self.data,
                                                self.param)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_500_INTERNAL_SERVER_ERROR)

    def test_patch_returns_200_when_data_are_valid_with_authorized_user(self):
        # Arrange
        user = create_mock_user('0', True, True)
        self.param = {
            'pk': self.fixture.data_1.id
        }

        self.data = {
            'name': 'test_',
            'filename': 'test_filename_',
        }

        # Act
        response = RequestMock.do_request_patch(xsl_views.XslTransformationDetail.as_view(),
                                                user,
                                                self.data,
                                                self.param)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestPostXslTransformationTransform(MongoIntegrationBaseTestCase):
    fixture = fixture_data

    def setUp(self):
        super(TestPostXslTransformationTransform, self).setUp()
        self.data = None

    def test_post_raise_error_500_if_xslt_does_not_exist(self):
        # Arrange
        user = create_mock_user('0')

        self.data = {
            'xml_content': 'xml_content',
            'xslt_name': 'xslt_name',
        }

        # Act
        response = RequestMock.do_request_post(xsl_views.XslTransformationTransform.as_view(),
                                               user,
                                               self.data)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_500_INTERNAL_SERVER_ERROR)

    def test_post_return_status_200_if_xml_and_xslt_name_are_valid_parameters(self):
        # Arrange
        user = create_mock_user('0')

        self.data = {
            'xml_content': '<test></test>',
            'xslt_name': 'name_1',
        }

        # Act
        response = RequestMock.do_request_post(xsl_views.XslTransformationTransform.as_view(),
                                               user,
                                               self.data)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)



"""REST views for the template version manager API
"""

from django.http import Http404
from django.utils.decorators import method_decorator
from rest_framework import status
from rest_framework.exceptions import ValidationError
from rest_framework.response import Response
from rest_framework.views import APIView

from core_main_app.commons import exceptions as exceptions
from core_main_app.components.template_version_manager import api as template_version_manager_api
from core_main_app.components.version_manager import api as version_manager_api
from core_main_app.rest.template_version_manager.abstract_views import AbstractTemplateVersionManagerList, \
    AbstractTemplateList, AbstractStatusTemplateVersion, AbstractStatusTemplateVersionManager, \
    AbstractTemplateVersionManagerDetail
from core_main_app.rest.template_version_manager.serializers import TemplateVersionManagerSerializer, \
    CreateTemplateSerializer
from core_main_app.utils.decorators import api_staff_member_required


class GlobalTemplateVersionManagerList(AbstractTemplateVersionManagerList):
    """ List all global template version managers.
    """

    def get_template_version_managers(self):
        """ Get global template version managers

        Returns:

        """
        return template_version_manager_api.get_global_version_managers()


class UserTemplateVersionManagerList(AbstractTemplateVersionManagerList):
    """ List all user template version managers.
    """

    def get_template_version_managers(self):
        """ Get all user template version managers.

        Returns:

        """
        return template_version_manager_api.get_all_by_user_id(user_id=str(self.request.user.id))


class TemplateVersionManagerDetail(APIView):
    """ Retrieve a template version manager.
    """
    def get_object(self, pk):
        """ Get template version manager from db

        Args:
            pk:

        Returns:

        """
        try:
            return version_manager_api.get(pk)
        except exceptions.DoesNotExist:
            raise Http404

    def get(self, request, pk):
        """ Retrieve template version manager

        Args:
            request:
            pk:

        Returns:

        """
        try:
            # Get object
            template_version_manager_object = self.get_object(pk)

            # Serialize object
            serializer = TemplateVersionManagerSerializer(template_version_manager_object)

            # Return response
            return Response(serializer.data)
        except Http404:
            content = {'message': 'Template Version Manager not found.'}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {'message': api_exception.message}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


class TemplateVersion(AbstractTemplateVersionManagerDetail):
    """ Create a version.
    """
    def post(self, request, pk):
        """ Create a version.

        POST data
        {
        "filename": "filename",
        "content": "<xs:schema xmlns:xs='http://www.w3.org/2001/XMLSchema'><xs:element name='root'/></xs:schema>"
        }

        Note: "dependencies"= json.dumps({"schemaLocation1": "id1" ,"schemaLocation2":"id2"})

        Args:
            request:

        Returns:

        """
        try:
            # Get object
            template_version_manager_object = self.get_object(pk)

            # Build serializers
            template_serializer = CreateTemplateSerializer(data=request.data)

            # Validate data
            template_serializer.is_valid(True)

            # Save data
            template_serializer.save(template_version_manager=template_version_manager_object)

            return Response(template_serializer.data, status=status.HTTP_201_CREATED)
        except Http404:
            content = {'message': 'Template Version Manager not found.'}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except ValidationError as validation_exception:
            content = {'message': validation_exception.detail}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except Exception as api_exception:
            content = {'message': api_exception.message}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


class UserTemplateList(AbstractTemplateList):
    """ Create a user template (owner is user)
    """
    def post(self, request):
        """ Create a user template.

        Args:
            request:

        Returns:

        """
        return super(UserTemplateList, self).post(request)

    def get_user(self):
        return str(self.request.user.id)


class GlobalTemplateList(AbstractTemplateList):
    """ Create a global template (owner is None)
    """
    @method_decorator(api_staff_member_required())
    def post(self, request):
        """ Create a global template.

        Args:
            request:

        Returns:

        """
        return super(GlobalTemplateList, self).post(request)

    def get_user(self):
        return None


class CurrentTemplateVersion(AbstractStatusTemplateVersion):
    """ Set current.
    """
    def status_update(self, template_object):
        """ Update status to current

        Args:
            template_object:

        Returns:

        """
        return version_manager_api.set_current(template_object)


class DisableTemplateVersion(AbstractStatusTemplateVersion):
    """ Set disabled.
    """
    def status_update(self, template_object):
        """ Update status to disabled

        Args:
            template_object:

        Returns:

        """
        return version_manager_api.disable_version(template_object)


class RestoreTemplateVersion(AbstractStatusTemplateVersion):
    """ Set restored
    """
    def status_update(self, template_object):
        """ Update status to restored

        Args:
            template_object:

        Returns:

        """
        return version_manager_api.restore_version(template_object)


class DisableTemplateVersionManager(AbstractStatusTemplateVersionManager):
    """ Set disabled.
    """

    def status_update(self, template_version_manager_object):
        """ Set disabled.

        Args:
            template_version_manager_object:

        Returns:

        """
        version_manager_api.disable(template_version_manager_object)


class RestoreTemplateVersionManager(AbstractStatusTemplateVersionManager):
    """ Set restored.
    """

    def status_update(self, template_version_manager_object):
        """ Set restored.

        Args:
            template_version_manager_object:

        Returns:

        """
        version_manager_api.restore(template_version_manager_object)



""" Urls utils
"""

import re

from django.urls import reverse


def get_template_download_pattern():
    """ Return regex pattern to match an url to download a template.

    Returns:

    """
    # build django url to download a template
    download_template_url = reverse('core_main_app_rest_template_download', kwargs={'pk': 'template_id'})
    # make url a regex
    download_template_regex = download_template_url.replace('template_id', '(?P<pk>\w+)')
    # compile regex
    pattern = re.compile(download_template_regex)

    return pattern



""" System api
"""
from core_main_app.components.data.models import Data


def get_data_by_id(data_id):
    """ Return data object with the given id.

        Parameters:
            data_id:

        Returns: data object
    """
    return Data.get_by_id(data_id)



""" Fixtures files for Group
"""

from core_main_app.components.group import api as group_api
from core_main_app.utils.integration_tests.fixture_interface import FixtureInterface


class GroupFixtures(FixtureInterface):
    """ Group Fixture
    """

    def insert_data(self):
        pass

    @staticmethod
    def create_group(name="name"):
        group, created = group_api.get_or_create(name=name)
        return group



""" Integration Test for Template Version Manager Rest API
"""
import json
import unittest

from django.test import override_settings
from django.urls import reverse
from rest_framework import status

from core_main_app.components.template import api as template_api
from core_main_app.components.version_manager import api as vm_api
from core_main_app.rest.template_version_manager import views
from core_main_app.utils.integration_tests.integration_base_test_case import \
    MongoIntegrationBaseTestCase
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock
from tests.components.template_version_manager.fixtures.fixtures import TemplateVersionManagerFixtures

fixture_template = TemplateVersionManagerFixtures()


class TestGlobalTemplateVersionManagerList(MongoIntegrationBaseTestCase):
    fixture = fixture_template

    def setUp(self):
        super(TestGlobalTemplateVersionManagerList, self).setUp()

    def test_get_returns_http_200(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_get(views.GlobalTemplateVersionManagerList.as_view(), user)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_get_returns_all_global_tvm(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_get(views.GlobalTemplateVersionManagerList.as_view(), user)

        # Assert
        self.assertEqual(len(response.data), 1)

    def test_get_returned_tvm_are_global(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_get(views.GlobalTemplateVersionManagerList.as_view(), user)

        # Assert
        self.assertEqual(response.data[0]['user'], None)

    def test_get_filtered_by_correct_title_returns_tvm(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_get(views.GlobalTemplateVersionManagerList.as_view(),
                                              user,
                                              data={'title': self.fixture.template_vm_1.title})

        # Assert
        self.assertEqual(len(response.data), 1)

    def test_get_filtered_by_incorrect_title_returns_no_tvm(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_get(views.GlobalTemplateVersionManagerList.as_view(),
                                              user,
                                              data={'title': 'bad title'})

        # Assert
        self.assertEqual(len(response.data), 0)

    def test_get_filtered_by_expected_is_disabled_returns_tvm(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_get(views.GlobalTemplateVersionManagerList.as_view(),
                                              user,
                                              data={'is_disabled': self.fixture.template_vm_1.is_disabled})

        # Assert
        self.assertEqual(len(response.data), 1)

    def test_get_filtered_by_incorrect_is_disabled_returns_no_tvm(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_get(views.GlobalTemplateVersionManagerList.as_view(),
                                              user,
                                              data={'is_disabled': not self.fixture.template_vm_1.is_disabled})

        # Assert
        self.assertEqual(len(response.data), 0)


class TestUserTemplateVersionManagerList(MongoIntegrationBaseTestCase):
    fixture = fixture_template

    def setUp(self):
        super(TestUserTemplateVersionManagerList, self).setUp()

    def test_get_returns_http_200(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_get(views.UserTemplateVersionManagerList.as_view(), user)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_get_returns_all_user_tvm(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_get(views.UserTemplateVersionManagerList.as_view(), user)

        # Assert
        self.assertEqual(len(response.data), 1)

    def test_get_returned_tvm_are_from_user(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_get(views.UserTemplateVersionManagerList.as_view(), user)

        # Assert
        self.assertEqual(response.data[0]['user'], '1')

    def test_get_filtered_by_correct_title_returns_tvm(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_get(views.UserTemplateVersionManagerList.as_view(),
                                              user,
                                              data={'title': self.fixture.template_vm_2.title})

        # Assert
        self.assertEqual(len(response.data), 1)

    def test_get_filtered_by_incorrect_title_returns_no_tvm(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_get(views.UserTemplateVersionManagerList.as_view(),
                                              user,
                                              data={'title': 'bad title'})

        # Assert
        self.assertEqual(len(response.data), 0)

    def test_get_filtered_by_expected_is_disabled_returns_tvm(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_get(views.UserTemplateVersionManagerList.as_view(),
                                              user,
                                              data={'is_disabled': self.fixture.template_vm_2.is_disabled})

        # Assert
        self.assertEqual(len(response.data), 1)

    def test_get_filtered_by_incorrect_is_disabled_returns_no_tvm(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_get(views.UserTemplateVersionManagerList.as_view(),
                                              user,
                                              data={'is_disabled': not self.fixture.template_vm_2.is_disabled})

        # Assert
        self.assertEqual(len(response.data), 0)


class TestTemplateVersionManagerDetail(MongoIntegrationBaseTestCase):
    fixture = fixture_template

    def setUp(self):
        super(TestTemplateVersionManagerDetail, self).setUp()

    def test_get_returns_http_200(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_get(views.TemplateVersionManagerDetail.as_view(),
                                              user,
                                              param={'pk': self.fixture.template_vm_1.id})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_get_returns_tvm(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_get(views.TemplateVersionManagerDetail.as_view(),
                                              user,
                                              param={'pk': self.fixture.template_vm_1.id})

        # Assert
        self.assertEqual(response.data['title'], self.fixture.template_vm_1.title)

    def test_get_wrong_id_returns_http_404(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_get(views.TemplateVersionManagerDetail.as_view(),
                                              user,
                                              param={'pk': '507f1f77bcf86cd799439011'})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)


class TestTemplateVersion(MongoIntegrationBaseTestCase):
    fixture = fixture_template

    def setUp(self):
        super(TestTemplateVersion, self).setUp()
        self.data = {"filename": "filename",
                     "content": "<xs:schema xmlns:xs='http://www.w3.org/2001/XMLSchema'>"
                                "<xs:element name='root'/></xs:schema>"}

    def post_returns_http_201(self):
        # Arrange
        user = create_mock_user('1', is_superuser=True)

        # Act
        response = RequestMock.do_request_post(views.TemplateVersion.as_view(),
                                               user,
                                               data=self.data,
                                               param={'pk': str(self.fixture.template_vm_1.id)})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    def post_wrong_id_returns_http_404(self):
        # Arrange
        user = create_mock_user('1', is_superuser=True)

        # Act
        response = RequestMock.do_request_post(views.TemplateVersion.as_view(),
                                               user,
                                               data=self.data,
                                               param={'pk': '507f1f77bcf86cd799439011'})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def post_to_other_user_returns_http_403(self):
        # Arrange
        user = create_mock_user('2')

        # Act
        response = RequestMock.do_request_post(views.TemplateVersion.as_view(),
                                               user,
                                               data=self.data,
                                               param={'pk': str(self.fixture.template_vm_2.id)})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def post_to_other_user_as_superuser_returns_http_201(self):
        # Arrange
        user = create_mock_user('2', is_superuser=True)

        # Act
        response = RequestMock.do_request_post(views.TemplateVersion.as_view(),
                                               user,
                                               data=self.data,
                                               param={'pk': str(self.fixture.template_vm_2.id)})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    def post_to_global_as_superuser_returns_http_201(self):
        # Arrange
        user = create_mock_user('2', is_superuser=True)

        # Act
        response = RequestMock.do_request_post(views.TemplateVersion.as_view(),
                                               user,
                                               data=self.data,
                                               param={'pk': str(self.fixture.template_vm_1.id)})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    def post_to_global_as_user_returns_http_403(self):
        # Arrange
        user = create_mock_user('2')

        # Act
        response = RequestMock.do_request_post(views.TemplateVersion.as_view(),
                                               user,
                                               data=self.data,
                                               param={'pk': str(self.fixture.template_vm_1.id)})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)


class TestUserTemplateList(MongoIntegrationBaseTestCase):
    fixture = fixture_template

    def setUp(self):
        super(TestUserTemplateList, self).setUp()
        self.data = {"title": "title",
                     "filename": "filename",
                     "content": "<xs:schema xmlns:xs='http://www.w3.org/2001/XMLSchema'>"
                                "<xs:element name='root'/></xs:schema>"}

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_post_returns_http_201(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_post(views.UserTemplateList.as_view(),
                                               user,
                                               data=self.data)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_post_owner_is_user(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_post(views.UserTemplateList.as_view(),
                                               user,
                                               data=self.data)

        # get template version manager from posted template
        template_id = response.data['id']
        template_object = template_api.get(template_id)
        template_version_manager = vm_api.get_from_version(template_object)

        # Assert
        self.assertEqual(template_version_manager.user, user.id)

    def test_post_template_name_already_exists_returns_http_400(self):
        # Arrange
        user = create_mock_user('1')
        self.data['title'] = self.fixture.template_vm_1.title

        # Act
        response = RequestMock.do_request_post(views.UserTemplateList.as_view(),
                                               user,
                                               data=self.data)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_post_template_with_correct_dependency_returns_http_201(self):
        # Arrange
        user = create_mock_user('1')
        self.data['content'] = "<xs:schema xmlns:xs='http://www.w3.org/2001/XMLSchema'>" \
                               "<xs:include schemaLocation='template1_1.xsd'/>" \
                               "<xs:element name='root'/></xs:schema>"

        self.data['dependencies_dict'] = json.dumps({"template1_1.xsd": str(self.fixture.template_1_1.id)})

        # Act
        response = RequestMock.do_request_post(views.UserTemplateList.as_view(),
                                               user,
                                               data=self.data)

        # Assert
        # FIXME: unable to download self dependency because server not running
        # self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        expected_download_url = reverse("core_main_app_rest_template_download",
                                        kwargs={'pk': self.fixture.template_1_1.id})
        self.assertTrue(expected_download_url in response.data['message'])

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_post_template_with_incorrect_dependency_schemaLocation_returns_http_400(self):
        # Arrange
        user = create_mock_user('1')
        self.data['content'] = "<xs:schema xmlns:xs='http://www.w3.org/2001/XMLSchema'>" \
                               "<xs:include schemaLocation='template1_1.xsd'/>" \
                               "<xs:element name='root'/></xs:schema>"

        self.data['dependencies_dict'] = json.dumps({"test.xsd": str(self.fixture.template_1_1.id)})

        # Act
        response = RequestMock.do_request_post(views.UserTemplateList.as_view(),
                                               user,
                                               data=self.data)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_post_template_with_incorrect_dependency_id_returns_http_400(self):
        # Arrange
        user = create_mock_user('1')
        self.data['content'] = "<xs:schema xmlns:xs='http://www.w3.org/2001/XMLSchema'>" \
                               "<xs:include schemaLocation='template1_1.xsd'/>" \
                               "<xs:element name='root'/></xs:schema>"

        self.data['dependencies_dict'] = json.dumps({"template1_1.xsd": "bad_id"})

        # Act
        response = RequestMock.do_request_post(views.UserTemplateList.as_view(),
                                               user,
                                               data=self.data)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)


class TestGlobalTemplateList(MongoIntegrationBaseTestCase):
    fixture = fixture_template

    def setUp(self):
        super(TestGlobalTemplateList, self).setUp()
        self.data = {"title": "title",
                     "filename": "filename",
                     "content": "<xs:schema xmlns:xs='http://www.w3.org/2001/XMLSchema'>"
                                "<xs:element name='root'/></xs:schema>"}

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_post_returns_http_201_if_user_is_staff(self):
        # Arrange
        user = create_mock_user('1', is_staff=True)

        # Act
        response = RequestMock.do_request_post(views.GlobalTemplateList.as_view(),
                                               user,
                                               data=self.data)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_post_returns_http_201_if_user_is_superuser(self):
        # Arrange
        user = create_mock_user('1', is_superuser=True)

        # Act
        response = RequestMock.do_request_post(views.GlobalTemplateList.as_view(),
                                               user,
                                               data=self.data)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_post_returns_http_403_if_user_does_not_have_permission(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_post(views.GlobalTemplateList.as_view(),
                                               user,
                                               data=self.data)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_post_owner_is_global(self):
        # Arrange
        user = create_mock_user('1', is_staff=True)

        # Act
        response = RequestMock.do_request_post(views.GlobalTemplateList.as_view(),
                                               user,
                                               data=self.data)

        # get template version manager from posted template
        template_id = response.data['id']
        template_object = template_api.get(template_id)
        template_version_manager = vm_api.get_from_version(template_object)

        # Assert
        self.assertEqual(template_version_manager.user, None)


class TestCurrentTemplateVersion(MongoIntegrationBaseTestCase):
    fixture = fixture_template

    def test_patch_returns_http_200(self):
        # Arrange
        user = create_mock_user('1', is_staff=True)

        # Act
        response = RequestMock.do_request_patch(views.CurrentTemplateVersion.as_view(),
                                                user,
                                                param={'pk': str(self.fixture.template_1_1.id)})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_patch_wrong_id_returns_http_404(self):
        # Arrange
        user = create_mock_user('1', is_staff=True)

        # Act
        response = RequestMock.do_request_patch(views.CurrentTemplateVersion.as_view(),
                                                user,
                                                param={'pk': '507f1f77bcf86cd799439011'})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_patch_disabled_returns_http_400(self):
        # Arrange
        user = create_mock_user('1', is_staff=True)

        # Act
        response = RequestMock.do_request_patch(views.CurrentTemplateVersion.as_view(),
                                                user,
                                                param={'pk': str(self.fixture.template_1_2.id)})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_patch_global_as_user_returns_http_403(self):
        # Arrange
        user = create_mock_user('2')

        # Act
        response = RequestMock.do_request_patch(views.CurrentTemplateVersion.as_view(),
                                                user,
                                                param={'pk': str(self.fixture.template_1_1.id)})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_patch_other_user_returns_http_403(self):
        # Arrange
        user = create_mock_user('2')

        # Act
        response = RequestMock.do_request_patch(views.CurrentTemplateVersion.as_view(),
                                                user,
                                                param={'pk': str(self.fixture.template_2_1.id)})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)


class TestDisableTemplateVersion(MongoIntegrationBaseTestCase):
    fixture = fixture_template

    def test_patch_returns_http_200(self):
        # Arrange
        user = create_mock_user('1', is_staff=True)

        # Act
        response = RequestMock.do_request_patch(views.DisableTemplateVersion.as_view(),
                                                user,
                                                param={'pk': str(self.fixture.template_1_1.id)})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_patch_wrong_id_returns_http_404(self):
        # Arrange
        user = create_mock_user('1', is_staff=True)

        # Act
        response = RequestMock.do_request_patch(views.DisableTemplateVersion.as_view(),
                                                user,
                                                param={'pk': '507f1f77bcf86cd799439011'})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_patch_already_disabled_returns_http_200(self):
        # Arrange
        user = create_mock_user('1', is_staff=True)

        # Act
        response = RequestMock.do_request_patch(views.DisableTemplateVersion.as_view(),
                                                user,
                                                param={'pk': str(self.fixture.template_1_2.id)})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_patch_current_returns_http_400(self):
        # Arrange
        user = create_mock_user('1', is_staff=True)

        # Act
        response = RequestMock.do_request_patch(views.DisableTemplateVersion.as_view(),
                                                user,
                                                param={'pk': str(self.fixture.template_1_3.id)})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_patch_global_as_user_returns_http_403(self):
        # Arrange
        user = create_mock_user('2')

        # Act
        response = RequestMock.do_request_patch(views.DisableTemplateVersion.as_view(),
                                                user,
                                                param={'pk': str(self.fixture.template_1_1.id)})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_patch_other_user_returns_http_403(self):
        # Arrange
        user = create_mock_user('2')

        # Act
        response = RequestMock.do_request_patch(views.DisableTemplateVersion.as_view(),
                                                user,
                                                param={'pk': str(self.fixture.template_2_1.id)})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)


class TestRestoreTemplateVersion(MongoIntegrationBaseTestCase):
    fixture = fixture_template

    def test_patch_returns_http_200(self):
        # Arrange
        user = create_mock_user('1', is_staff=True)

        # Act
        response = RequestMock.do_request_patch(views.RestoreTemplateVersion.as_view(),
                                                user,
                                                param={'pk': str(self.fixture.template_1_2.id)})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_patch_wrong_id_returns_http_404(self):
        # Arrange
        user = create_mock_user('1', is_staff=True)

        # Act
        response = RequestMock.do_request_patch(views.RestoreTemplateVersion.as_view(),
                                                user,
                                                param={'pk': '507f1f77bcf86cd799439011'})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_patch_not_disabled_returns_http_400(self):
        # Arrange
        user = create_mock_user('1', is_staff=True)

        # Act
        response = RequestMock.do_request_patch(views.RestoreTemplateVersion.as_view(),
                                                user,
                                                param={'pk': str(self.fixture.template_1_1.id)})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_patch_global_as_user_returns_http_403(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_patch(views.RestoreTemplateVersion.as_view(),
                                                user,
                                                param={'pk': str(self.fixture.template_1_2.id)})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_patch_other_user_returns_http_403(self):
        # Arrange
        user = create_mock_user('2')

        # Act
        response = RequestMock.do_request_patch(views.RestoreTemplateVersion.as_view(),
                                                user,
                                                param={'pk': str(self.fixture.template_2_1.id)})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)


class TestDisableTemplateVersionManager(MongoIntegrationBaseTestCase):
    fixture = fixture_template

    def test_patch_returns_http_200(self):
        # Arrange
        user = create_mock_user('1', is_staff=True)

        # Act
        response = RequestMock.do_request_patch(views.DisableTemplateVersionManager.as_view(),
                                                user,
                                                param={'pk': str(self.fixture.template_vm_1.id)})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_patch_wrong_id_returns_http_404(self):
        # Arrange
        user = create_mock_user('1', is_staff=True)

        # Act
        response = RequestMock.do_request_patch(views.DisableTemplateVersionManager.as_view(),
                                                user,
                                                param={'pk': '507f1f77bcf86cd799439011'})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_patch_already_disabled_returns_http_200(self):
        # Arrange
        user = create_mock_user('1', is_staff=True)
        self.fixture.template_vm_1.is_disabled = True

        # Act
        response = RequestMock.do_request_patch(views.DisableTemplateVersionManager.as_view(),
                                                user,
                                                param={'pk': str(self.fixture.template_vm_1.id)})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_patch_global_as_user_returns_http_403(self):
        # Arrange
        user = create_mock_user('2')

        # Act
        response = RequestMock.do_request_patch(views.DisableTemplateVersionManager.as_view(),
                                                user,
                                                param={'pk': str(self.fixture.template_vm_1.id)})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_patch_other_user_returns_http_403(self):
        # Arrange
        user = create_mock_user('2')

        # Act
        response = RequestMock.do_request_patch(views.DisableTemplateVersionManager.as_view(),
                                                user,
                                                param={'pk': str(self.fixture.template_vm_2.id)})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)


class TestRestoreTemplateVersionManager(MongoIntegrationBaseTestCase):
    fixture = fixture_template

    def test_patch_returns_http_200(self):
        # Arrange
        user = create_mock_user('1', is_staff=True)
        self.fixture.template_vm_1.is_disabled = True

        # Act
        response = RequestMock.do_request_patch(views.RestoreTemplateVersionManager.as_view(),
                                                user,
                                                param={'pk': str(self.fixture.template_vm_1.id)})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_patch_wrong_id_returns_http_404(self):
        # Arrange
        user = create_mock_user('1', is_staff=True)

        # Act
        response = RequestMock.do_request_patch(views.RestoreTemplateVersionManager.as_view(),
                                                user,
                                                param={'pk': '507f1f77bcf86cd799439011'})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_patch_not_disabled_returns_http_200(self):
        # Arrange
        user = create_mock_user('1', is_staff=True)

        # Act
        response = RequestMock.do_request_patch(views.RestoreTemplateVersionManager.as_view(),
                                                user,
                                                param={'pk': str(self.fixture.template_vm_1.id)})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_patch_global_as_user_returns_http_403(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_patch(views.RestoreTemplateVersionManager.as_view(),
                                                user,
                                                param={'pk': str(self.fixture.template_vm_1.id)})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_patch_other_user_returns_http_403(self):
        # Arrange
        user = create_mock_user('2')

        # Act
        response = RequestMock.do_request_patch(views.RestoreTemplateVersionManager.as_view(),
                                                user,
                                                param={'pk': str(self.fixture.template_vm_2.id)})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)



"""Serializers used throughout the data Rest API
"""
from rest_framework import serializers
from rest_framework_mongoengine.serializers import DocumentSerializer

import core_main_app.components.data.api as data_api
from core_main_app.components.data.models import Data


class XMLContentField(serializers.Field):
    """
    XML content is decoded when retrieved - not supported by CharField
    """
    def to_representation(self, obj):
        return obj.decode('utf-8')

    def to_internal_value(self, data):
        return data


class DataSerializer(DocumentSerializer):
    """ Data serializer
    """
    xml_content = XMLContentField()

    class Meta:
        """ Meta
        """
        model = Data
        fields = ["id",
                  "template",
                  "user_id",
                  "title",
                  "xml_content",
                  "last_modification_date"]
        read_only_fields = ('id', 'user_id', 'last_modification_date', )

    def create(self, validated_data):
        """
        Create and return a new `Data` instance, given the validated data.
        """
        # Create data
        instance = Data(
            template=validated_data['template'],
            title=validated_data['title'],
            user_id=str(validated_data['user'].id),
        )
        # Set xml content
        instance.xml_content = validated_data['xml_content']
        # Save the data
        return data_api.upsert(instance, validated_data['user'])

    def update(self, instance, validated_data):
        """
        Update and return an existing `Data` instance, given the validated data.
        """
        instance.title = validated_data.get('title', instance.title)
        instance.xml_content = validated_data.get('xml_content', instance.xml_content)
        return data_api.upsert(instance, validated_data['user'])


# FIXME: Should use in the future an serializer with dynamic fields (init depth with parameter for example)
class DataWithTemplateInfoSerializer(DocumentSerializer):
    """ Data Full serializer
    """
    class Meta:
        """ Meta
        """
        model = Data
        depth = 2
        fields = ["id",
                  "template",
                  "user_id",
                  "title",
                  "xml_content",
                  "last_modification_date"]




from core_main_app.utils.databases.mongoengine_database import Database

SECRET_KEY = 'fake-key'

INSTALLED_APPS = [
    # Django apps
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sites',

    # Extra apps
    "password_policies",

    # Local apps
    "core_main_app",
    "tests",
]

# IN-MEMORY TEST DATABASE
DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": ":memory:",
        "USER": "",
        "PASSWORD": "",
        "HOST": "",
        "PORT": "",
    },
}


PASSWORD_HASHERS = (
    'django.contrib.auth.hashers.UnsaltedMD5PasswordHasher',
)

MOCK_DATABASE_NAME = 'db_mock'
MOCK_DATABASE_HOST = 'mongomock://localhost'

database = Database(MOCK_DATABASE_HOST, MOCK_DATABASE_NAME)
database.connect()



""" Integration Test for Data Rest API
"""
import json

from mock import patch
from rest_framework import status

from core_main_app.components.data.models import Data
from core_main_app.rest.data import views as data_rest_views
from core_main_app.utils.integration_tests.integration_base_test_case import \
    MongoIntegrationBaseTestCase
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock
from tests.components.data.fixtures.fixtures import DataFixtures, QueryDataFixtures

fixture_data = DataFixtures()
fixture_data_query = QueryDataFixtures()


class TestDataList(MongoIntegrationBaseTestCase):
    fixture = fixture_data

    def setUp(self):
        super(TestDataList, self).setUp()

    @patch.object(Data, 'xml_content')
    def test_get_returns_http_200(self, mock_xml_content):
        # Arrange
        user = create_mock_user('1')
        mock_xml_content.return_value = "content"

        # Act
        response = RequestMock.do_request_get(data_rest_views.DataList.as_view(), user)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_get_returns_all_user_data(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_get(data_rest_views.DataList.as_view(), user)

        # Assert
        self.assertEqual(len(response.data), 1)

    def test_get_filtered_by_correct_title_returns_data(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_get(data_rest_views.DataList.as_view(),
                                              user,
                                              data={'title': self.fixture.data_1.title})

        # Assert
        self.assertEqual(len(response.data), 1)

    def test_get_filtered_by_incorrect_title_returns_no_data(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_get(data_rest_views.DataList.as_view(),
                                              user,
                                              data={'title': 'bad title'})

        # Assert
        self.assertEqual(len(response.data), 0)

    def test_get_filtered_by_correct_template_returns_data(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_get(data_rest_views.DataList.as_view(),
                                              user,
                                              data={'template': self.fixture.data_1.template})

        # Assert
        self.assertEqual(len(response.data), 1)

    def test_get_filtered_by_incorrect_template_returns_no_data(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_get(data_rest_views.DataList.as_view(),
                                              user,
                                              data={'template': '507f1f77bcf86cd799439011'})

        # Assert
        self.assertEqual(len(response.data), 0)

    def test_post_data_missing_field_returns_http_400(self):
        # Arrange
        user = create_mock_user('1')
        mock_data = {'template': str(self.fixture.template.id),
                     'user_id': '1',
                     'xml_content': '<tag></tag>'}

        # Act
        response = RequestMock.do_request_post(data_rest_views.DataList.as_view(),
                                               user,
                                               data=mock_data)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_post_data_incorrect_template_returns_http_400(self):
        # Arrange
        user = create_mock_user('1')
        mock_data = {'template': '507f1f77bcf86cd799439011',
                     'user_id': '1',
                     'title': 'new data',
                     'xml_content': '<tag></tag>'}

        # Act
        response = RequestMock.do_request_post(data_rest_views.DataList.as_view(),
                                               user,
                                               data=mock_data)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)


class TestDataDetail(MongoIntegrationBaseTestCase):
    fixture = fixture_data

    def setUp(self):
        super(TestDataDetail, self).setUp()

    @patch.object(Data, 'xml_content')
    def test_get_returns_http_200(self, mock_xml_content):
        # Arrange
        user = create_mock_user('1')
        mock_xml_content.return_value = "content"

        # Act
        response = RequestMock.do_request_get(data_rest_views.DataDetail.as_view(),
                                              user,
                                              param={'pk': self.fixture.data_1.id})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(Data, 'xml_content')
    def test_get_returns_data(self, mock_xml_content):
        # Arrange
        user = create_mock_user('1')
        mock_xml_content.return_value = "content"

        # Act
        response = RequestMock.do_request_get(data_rest_views.DataDetail.as_view(),
                                              user,
                                              param={'pk': self.fixture.data_1.id})

        # Assert
        self.assertEqual(response.data['title'], self.fixture.data_1.title)

    @patch.object(Data, 'xml_content')
    def test_get_data_containing_ascii_returns_data(self, mock_xml_content):
        # Arrange
        user = create_mock_user('1')
        mock_xml_content.return_value = "\xc3te\xc3"

        # Act
        response = RequestMock.do_request_get(data_rest_views.DataDetail.as_view(),
                                              user,
                                              param={'pk': self.fixture.data_1.id})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_get_wrong_id_returns_http_404(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_get(data_rest_views.DataDetail.as_view(),
                                              user,
                                              param={'pk': '507f1f77bcf86cd799439011'})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_delete_wrong_id_returns_http_404(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_delete(data_rest_views.DataDetail.as_view(),
                                                 user,
                                                 param={'pk': '507f1f77bcf86cd799439011'})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_patch_wrong_id_returns_http_404(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_patch(data_rest_views.DataDetail.as_view(),
                                                user,
                                                param={'pk': '507f1f77bcf86cd799439011'})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_patch_wrong_template_returns_http_400(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_patch(data_rest_views.DataDetail.as_view(),
                                                user,
                                                data={'template': '507f1f77bcf86cd799439011'},
                                                param={'pk': self.fixture.data_1.id})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)


class TestDataDownload(MongoIntegrationBaseTestCase):
    fixture = fixture_data

    def setUp(self):
        super(TestDataDownload, self).setUp()

    def test_get_returns_http_200(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_get(data_rest_views.DataDownload.as_view(),
                                              user,
                                              param={'pk': self.fixture.data_1.id})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_get_wrong_id_returns_http_404(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_get(data_rest_views.DataDownload.as_view(),
                                              user,
                                              param={'pk': '507f1f77bcf86cd799439011'})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)


class TestExecuteLocalQueryView(MongoIntegrationBaseTestCase):
    fixture = fixture_data_query

    def setUp(self):
        super(TestExecuteLocalQueryView, self).setUp()
        # FIXME: unable to test paginated results (mocked queryset.count always returns 0)
        self.data = {"all": "true"}
        # create user with superuser access to skip access control
        self.user = create_mock_user('1', is_superuser=True)

    def test_post_query_one_data_returns_http_200(self):
        # Arrange
        self.data.update({"query": "{\"root.element\": \"value\"}"})

        # Act
        response = RequestMock.do_request_post(data_rest_views.ExecuteLocalQueryView.as_view(),
                                               self.user,
                                               data=self.data)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_post_query_one_data_returns_one_data(self):
        # Arrange
        self.data.update({"query": "{\"root.element\": \"value\"}"})

        # Act
        response = RequestMock.do_request_post(data_rest_views.ExecuteLocalQueryView.as_view(),
                                               self.user,
                                               data=self.data)

        # Assert
        self.assertEqual(len(response.data), 1)

    def test_post_query_two_data_returns_two_data(self):
        # Arrange
        self.data.update({"query": "{\"$or\": [{\"root.element\": \"value\"}, {\"root.element\":\"value2\"}]}"})

        # Act
        response = RequestMock.do_request_post(data_rest_views.ExecuteLocalQueryView.as_view(),
                                               self.user,
                                               data=self.data)

        # Assert
        self.assertEqual(len(response.data), 2)

    def test_post_empty_query_filter_by_templates_returns_all_data_of_the_template(self):
        # Arrange
        self.data.update({"query": "{}",
                          "templates": '[{"id": "' + str(self.fixture.template.id) + '"}]'})

        # Act
        response = RequestMock.do_request_post(data_rest_views.ExecuteLocalQueryView.as_view(),
                                               self.user,
                                               data=self.data)

        # Assert
        self.assertEqual(len(response.data), 2)

    def test_post_query_filtered_by_templates_returns_one_data(self):
        # Arrange
        self.data.update({"query": "{\"root.element\": \"value\"}",
                          "templates": '[{"id": "' + str(self.fixture.template.id) + '"}]'})

        # Act
        response = RequestMock.do_request_post(data_rest_views.ExecuteLocalQueryView.as_view(),
                                               self.user,
                                               data=self.data)

        # Assert
        self.assertEqual(len(response.data), 1)

    def test_post_query_filtered_by_wrong_template_id_returns_no_data(self):
        # Arrange
        self.data.update({"query": "{\"root.element\": \"value\"}",
                          "templates": '[{"id": "507f1f77bcf86cd799439011"}]'})

        # Act
        response = RequestMock.do_request_post(data_rest_views.ExecuteLocalQueryView.as_view(),
                                               self.user,
                                               data=self.data)

        # Assert
        self.assertEqual(len(response.data), 0)



""" Common forms
"""
from django import forms

from core_main_app.commons.validators import BlankSpacesValidator


class RenameForm(forms.Form):
    """ Rename form
    """
    field = forms.CharField(label='',
                            widget=forms.TextInput(attrs={'class': 'form-control', 'placeholder': 'Type the new name'}),
                            validators=[BlankSpacesValidator()])

    id = forms.CharField(widget=forms.HiddenInput(), required=False)



"""
    Common ajax
"""
import json

from django.contrib import messages
from django.core.urlresolvers import reverse_lazy
from django.http.response import HttpResponse, HttpResponseBadRequest, JsonResponse
from django.views.generic.edit import UpdateView, CreateView, DeleteView

from core_main_app.commons import exceptions
from core_main_app.components.template import api as template_api
from core_main_app.components.template_version_manager import api as template_version_manager_api
from core_main_app.components.template_version_manager.models import TemplateVersionManager
from core_main_app.components.version_manager import api as version_manager_api
from core_main_app.views.admin.forms import EditTemplateForm


class AddObjectModalView(CreateView):
    """ Common AddObjectModalView.
        Should be used with add.html and add.js.
    """
    template_name = 'core_main_app/common/commons/form.html'
    form_class = None
    model = None
    success_url = None
    success_message = None

    def form_invalid(self, form):
        # Get initial response
        response = super(AddObjectModalView, self).form_invalid(form)
        data = {
            'is_valid': False,
            'responseText': response.rendered_content
        }
        return JsonResponse(data)

    def form_valid(self, form):
        # This method is called when valid form data has been POSTed.
        # Call private save method
        # Populate self.object without committing to the database
        self.object = form.save(commit=False)
        self._save(form)
        if form.is_valid():
            data = {
                'is_valid': True,
                'url': self.get_success_url(),
            }

            if self.success_message:
                messages.success(self.request, self.success_message)

            return JsonResponse(data)
        else:
            return self.form_invalid(form)

    def _save(self, form):
        # Save treatment.
        super(AddObjectModalView, self).form_valid(form)

    @staticmethod
    def get_modal_html_path():
        return "core_main_app/common/modals/add_page_modal.html"

    @staticmethod
    def get_modal_js_path():
        return {"path": 'core_main_app/common/js/modals/add.js', "is_raw": False}


class EditObjectModalView(UpdateView):
    """ Common EditObjectModalView.
        Should be used with edit_page_modal.html and edit.js.
    """
    template_name = 'core_main_app/common/commons/form.html'
    form_class = None
    model = None
    success_url = None
    success_message = None

    def form_invalid(self, form):
        # Get initial response
        response = super(EditObjectModalView, self).form_invalid(form)
        data = {
            'is_valid': False,
            'responseText': response.rendered_content
        }
        return JsonResponse(data)

    def form_valid(self, form):
        # This method is called when valid form data has been POSTed.
        # Call private save method
        self._save(form)
        if form.is_valid():
            data = {
                'is_valid': True,
                'url': self.get_success_url(),
            }

            if self.success_message:
                messages.success(self.request, self.success_message)

            return JsonResponse(data)
        else:
            return self.form_invalid(form)

    def _save(self, form):
        # Save treatment.
        super(EditObjectModalView, self).form_valid(form)

    @staticmethod
    def get_modal_html_path():
        return "core_main_app/common/modals/edit_page_modal.html"

    @staticmethod
    def get_modal_js_path():
        return {"path": 'core_main_app/common/js/modals/edit.js', "is_raw": False}


class DeleteObjectModalView(DeleteView):
    """ Common DeleteObjectModalView.
        Should be used with delete_page_modal.html and delete.js.
    """
    template_name = 'core_main_app/common/commons/form_delete.html'
    model = None
    field_for_name = None
    success_url = None
    success_message = None

    def delete(self, request, *args, **kwargs):
        """
        Delete method.
        """
        try:
            self.object = self.get_object()
            self._delete(request, *args, **kwargs)

            if self.success_message:
                messages.success(self.request, self.success_message)
        except Exception, e:
            messages.error(self.request, e.message)

        data = {'url': self.get_success_url()}
        return JsonResponse(data)

    def _delete(self, request, *args, **kwargs):
        """
        Delete treatment.
        """
        # Delete treatment.
        super(DeleteObjectModalView, self).delete(request, *args, **kwargs)

    def _get_object_name(self):
        """
        Get object name
        """
        object_name = ''
        if self.object:
            if self.field_for_name:
                object_name = getattr(self.object, self.field_for_name, str(self.object))
            else:
                object_name = str(self.object)

        return object_name

    def get_context_data(self, **kwargs):
        # Call the base implementation first to get a context
        context = super(DeleteObjectModalView, self).get_context_data(**kwargs)
        # Add the object representation
        context['object_name'] = self._get_object_name()

        return context

    @staticmethod
    def get_modal_html_path():
        return "core_main_app/common/modals/delete_page_modal.html"

    @staticmethod
    def get_modal_js_path():
        return {"path": 'core_main_app/common/js/modals/delete.js', "is_raw": False}


class EditTemplateVersionManagerView(EditObjectModalView):
    form_class = EditTemplateForm
    model = TemplateVersionManager
    success_url = reverse_lazy("admin:core_main_app_templates")
    success_message = 'Name edited with success.'

    def _save(self, form):
        # Save treatment.
        try:
            template_version_manager_api.edit_title(self.object, form.cleaned_data.get('title'))
        except exceptions.NotUniqueError:
            form.add_error(None, "An object with the same name already exists. Please choose "
                                 "another name.")
        except Exception, e:
            form.add_error(None, e.message)


def disable_version_manager(request):
    """Disable a version manager.

    Args:
        request:

    Returns:

    """
    try:
        version_manager = version_manager_api.get(request.GET['id'])
        version_manager_api.disable(version_manager)
    except Exception, e:
        return HttpResponseBadRequest(e.message, content_type='application/javascript')

    return HttpResponse(json.dumps({}), content_type='application/javascript')


def restore_version_manager(request):
    """Restore a disabled version manager.

    Args:
        request:

    Returns:

    """
    try:
        version_manager = version_manager_api.get(request.GET['id'])
        version_manager_api.restore(version_manager)
    except Exception, e:
        return HttpResponseBadRequest(e.message, content_type='application/javascript')

    return HttpResponse(json.dumps({}), content_type='application/javascript')


def disable_template_version_from_version_manager(request):
    """ Disable a template version of a version manager.

    Args:
        request:

    Returns:

    """
    try:
        disable_version_of_version_manager(template_api.get(request.GET['id']))
    except Exception, e:
        return HttpResponseBadRequest(e.message, content_type='application/javascript')

    return HttpResponse(json.dumps({}), content_type='application/javascript')


def disable_version_of_version_manager(version):
    """ Disable a version of a version manager.

    Args:
        version:

    Returns:

    """
    try:
        version_manager_api.disable_version(version)
    except Exception, e:
        return HttpResponseBadRequest(e.message, content_type='application/javascript')


def restore_template_version_from_version_manager(request):
    """ Restore a disabled template version of a version manager.

    Args:
        request:

    Returns:

    """
    try:
        restore_version_from_version_manager(template_api.get(request.GET['id']))
    except Exception, e:
        return HttpResponseBadRequest(e.message, content_type='application/javascript')

    return HttpResponse(json.dumps({}), content_type='application/javascript')


def restore_version_from_version_manager(version):
    """ Restore a disabled version of a version manager.

    Args:
        version:

    Returns:

    """
    try:
        version_manager_api.restore_version(version)
    except Exception, e:
        return HttpResponseBadRequest(e.message, content_type='application/javascript')


def set_current_template_version_from_version_manager(request):
    """ Set the current version of a template.

    Args:
        request:

    Returns:

    """
    try:
        set_current_version_from_version_manager(template_api.get(request.GET['id']))
    except Exception, e:
        return HttpResponseBadRequest(e.message, content_type='application/javascript')

    return HttpResponse(json.dumps({}), content_type='application/javascript')


def set_current_version_from_version_manager(version):
    """ Set the current version of a version manager.

    Args:
        version:

    Returns:

    """
    try:
        version_manager_api.set_current(version)
    except Exception, e:
        return HttpResponseBadRequest(e.message, content_type='application/javascript')




"""XSL Transformation tag
"""
from django import template
from os.path import join
from django.contrib.staticfiles import finders

from core_main_app.utils.file import read_file_content
from core_main_app.utils.xml import xsl_transform
from core_main_app.components.template_xsl_rendering import api as template_xsl_rendering_api
from core_main_app.commons import exceptions
register = template.Library()


class XSLType(object):
    type_list = "List"
    type_detail = "Detail"


@register.simple_tag(name='xsl_transform_list')
def render_xml_as_html(*args, **kwargs):
    """ Render an XML to HTML using the list xslt.
    Args:
        *args:
        **kwargs:

    Returns:
        HTML

    """
    xml_content = kwargs['xml_content']
    template_id = kwargs.get('template_id', None)
    template_hash = kwargs.get('template_hash', None)
    return _render_xml_as_html(xml_content, template_id, template_hash, XSLType.type_list)


@register.simple_tag(name='xsl_transform_detail')
def render_xml_as_html(*args, **kwargs):
    """ Render an XML to HTML using the detail xslt.
    Args:
        *args:
        **kwargs:

    Returns:
        HTML

    """
    xml_content = kwargs['xml_content']
    template_id = kwargs.get('template_id', None)
    template_hash = kwargs.get('template_hash', None)
    return _render_xml_as_html(xml_content, template_id, template_hash, XSLType.type_detail)


def _render_xml_as_html(xml_string, template_id=None, template_hash=None, xslt_type=XSLType.type_list):
    """ Render an XML to HTML according to an xslt type (list or detail).
    Args:
        xml_string:
        template_id:
        template_hash:
        xslt_type:

    Returns:
        HTML

    """
    try:
        try:
            if template_id:
                template_xsl_rendering = template_xsl_rendering_api.get_by_template_id(template_id)
            elif template_hash:
                template_xsl_rendering = template_xsl_rendering_api.get_by_template_hash(template_hash)
            else:
                raise Exception("No template information provided. Default xslt will be used.")

            if xslt_type == XSLType.type_list:
                xslt_string = template_xsl_rendering.list_xslt.content
            elif xslt_type == XSLType.type_detail:
                xslt_string = template_xsl_rendering.detail_xslt.content
            else:
                raise Exception("XSLT Type unknown. Default xslt will be used.")
        except (Exception, exceptions.DoesNotExist):
            default_xslt_path = finders.find(join('core_main_app', 'common', 'xsl', 'xml2html.xsl'))
            xslt_string = read_file_content(default_xslt_path)

        return xsl_transform(xml_string, xslt_string)
    except Exception:
        return xml_string



"""REST views for the template API
"""

from django.http import Http404
from rest_framework import status
from rest_framework.response import Response
from rest_framework.views import APIView

from core_main_app.commons import exceptions as exceptions
from core_main_app.components.template import api as template_api
from core_main_app.rest.template.serializers import TemplateSerializer
from core_main_app.utils.file import get_file_http_response


class TemplateDetail(APIView):
    """
    Retrieve a template.
    """
    def get_object(self, pk):
        """ Get template from db

        Args:
            pk:

        Returns:

        """
        try:
            return template_api.get(pk)
        except exceptions.DoesNotExist:
            raise Http404

    def get(self, request, pk):
        """ Retrieve template

        Args:
            request:
            pk:

        Returns:

        """
        try:
            # Get object
            template_object = self.get_object(pk)

            # Serialize object
            serializer = TemplateSerializer(template_object)

            # Return response
            return Response(serializer.data)
        except Http404:
            content = {'message': 'Template not found.'}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {'message': api_exception.message}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


class TemplateDownload(APIView):
    """
    Download a template.
    """
    def get_object(self, pk):
        """ Get template from db

        Args:
            pk:

        Returns:

        """
        try:
            return template_api.get(pk)
        except exceptions.DoesNotExist:
            raise Http404

    def get(self, request, pk):
        """ Retrieve template

        Args:
            request:
            pk:

        Returns:

        """
        try:
            # Get object
            template_object = self.get_object(pk)

            return get_file_http_response(template_object.content, template_object.filename, 'text/xsd', 'xsd')
        except Http404:
            content = {'message': 'Template not found.'}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {'message': api_exception.message}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)



""" Fixture interface
"""


class FixtureInterface(object):
    """ Represent the fixture interface with methods which must be implemented.
    """

    def insert_data(self):
        """ Insert fixture's data

        Returns:

        """
        raise NotImplementedError()



"""Utils for version manager components
"""


def get_latest_version_name(version_manager):
    """Get name for the latest version of version manager.

    Args:
        version_manager:

    Returns:

    """
    return get_version_name(version_manager.title, str(len(version_manager.versions)))


def get_version_name(title, version_number):
    """Get a version name from a title and a version number.

    Args:
        title:
        version_number:

    Returns:

    """
    return "{0} (Version {1})".format(title.encode('utf-8'), str(version_number))



"""
Template Version Manager model
"""
from mongoengine import errors as mongoengine_errors

from core_main_app.commons import exceptions
from core_main_app.components.version_manager.models import VersionManager


class TemplateVersionManager(VersionManager):
    """Manager of templates versions"""

    # TODO: see if better way to find _cls
    class_name = 'VersionManager.TemplateVersionManager'

    @staticmethod
    def get_global_version_managers(_cls=True):
        """Return all Template Version Managers with user set to None.

        Returns:
            _cls: if True, restricts to TemplateVersionManager _cls
        """
        queryset = super(TemplateVersionManager, TemplateVersionManager).get_global_version_managers()
        if _cls:
            queryset = queryset.filter(_cls=TemplateVersionManager.class_name).all()
        return queryset

    @staticmethod
    def get_active_global_version_manager(_cls=True):
        """ Return all active Version Managers with user set to None.

        Returns:

        """
        queryset = super(TemplateVersionManager, TemplateVersionManager).get_active_global_version_manager()
        if _cls:
            queryset = queryset.filter(_cls=TemplateVersionManager.class_name).all()
        return queryset

    @staticmethod
    def get_disable_global_version_manager(_cls=True):
        """ Return all disabled Version Managers with user set to None.

        Returns:

        """
        queryset = super(TemplateVersionManager, TemplateVersionManager).get_disable_global_version_manager()
        if _cls:
            queryset = queryset.filter(_cls=TemplateVersionManager.class_name).all()
        return queryset

    @staticmethod
    def get_active_version_manager_by_user_id(user_id, _cls=True):
        """ Return all active Version Managers with given user id.

        Returns:

        """
        queryset = super(TemplateVersionManager, TemplateVersionManager).get_active_version_manager_by_user_id(user_id)
        if _cls:
            queryset = queryset.filter(_cls=TemplateVersionManager.class_name).all()
        return queryset

    @staticmethod
    def get_disable_version_manager_by_user_id(user_id, _cls=True):
        """ Return all disabled Version Managers with given user id.

        Returns:

        """
        queryset = super(TemplateVersionManager, TemplateVersionManager).get_disable_version_manager_by_user_id(user_id)
        if _cls:
            queryset = queryset.filter(_cls=TemplateVersionManager.class_name).all()
        return queryset

    @staticmethod
    def get_by_version_id(version_id):
        """Get the template version manager containing the given version id.

        Args:
            version_id: version id.

        Returns:
            template version manager.

        """
        try:
            return TemplateVersionManager.objects.get(versions__contains=version_id)
        except mongoengine_errors.DoesNotExist as e:
            raise exceptions.DoesNotExist(e.message)
        except Exception as ex:
            raise exceptions.ModelError(ex.message)

    @staticmethod
    def get_all_by_version_ids(version_ids):
        """Get all template version managers by a list of version ids.

        Args:
            version_ids: list of version ids.

        Returns:
            List of template version managers.

        """
        return TemplateVersionManager.objects(versions__in=version_ids).all()

    @staticmethod
    def get_all_version_manager_except_user_id(user_id, _cls=True):
        """ Return all Version Managers of all users except user with given user id.

        Args:
            user_id:
            _cls:

        Returns:

        """
        queryset = super(TemplateVersionManager, TemplateVersionManager).get_all_version_manager_except_user_id(user_id)
        if _cls:
            queryset = queryset.filter(_cls=TemplateVersionManager.class_name).all()
        return queryset

    @staticmethod
    def get_all_version_manager_by_user_id(user_id, _cls=True):
        """ Return all Version Managers with given user id.

        Args:
            user_id:
            _cls:

        Returns:

        """
        queryset = super(TemplateVersionManager, TemplateVersionManager).get_all_version_manager_by_user_id(user_id)
        if _cls:
            queryset = queryset.filter(_cls=TemplateVersionManager.class_name).all()
        return queryset

    @staticmethod
    def get_all_version_manager(_cls=True):
        """ Return all Version Managers.

            Args:
                _cls:

            Returns:

            """
        queryset = super(TemplateVersionManager, TemplateVersionManager).get_all()
        if _cls:
            queryset = queryset.filter(_cls=TemplateVersionManager.class_name).all()
        return queryset

    def save_template_version_manager(self):
        """ Custom save.

        Returns:
            Saved Instance.

        """
        return super(TemplateVersionManager, self).save_version_manager()



""" Rights for core main app
"""

# Anonymous group
anonymous_group = "anonymous"

# Default group
default_group = "default"

# API rights
api_content_type = "api"
api_access = "api_access"

# Data
edit_data = "edit_data"
delete_data = "delete_data"

# Form
edit_form = "edit_form"
delete_form = "delete_form"

# Workspace
CAN_READ_CODENAME = "can_read_workspace_"
CAN_WRITE_CODENAME = "can_write_workspace_"

CAN_READ_NAME = "Can read workspace "
CAN_WRITE_NAME = "Can write workspace "

CONTENT_TYPE_APP_LABEL = "core_main_app"




""" discover rules for core main app
"""
import core_main_app.permissions.rights as rights


def init_rules(apps):
    """ Init of group and permissions for the application.
    If the anonymous group does not exist, creation of the group
    If the default group does not exist, creation of the group

    Returns:

    """
    try:
        group = apps.get_model("auth", "Group")

        # Get or Create the Group anonymous
        group.objects.get_or_create(name=rights.anonymous_group)

        # Get or Create the default group
        group.objects.get_or_create(name=rights.default_group)

    except Exception, e:
        print('ERROR : Impossible to init the rules : ' + e.message)



""" Common custom fields
"""
from django import forms
from core_main_app import settings
from password_policies.forms.fields import PasswordPoliciesField
from core_main_app.commons.validators import UpperCaseLetterCountValidator, LowerCaseLetterCountValidator


class CustomPasswordPoliciesField(PasswordPoliciesField):
    """ A form field that validates a password using :doc:`validators`.
    """

    def __init__(self, *args, **kwargs):
        if "widget" not in kwargs:
            kwargs["widget"] = forms.PasswordInput(render_value=False)
        kwargs["min_length"] = settings.PASSWORD_MIN_LENGTH
        self.default_validators.append(UpperCaseLetterCountValidator(settings.PASSWORD_MIN_UPPERCASE_LETTERS))
        self.default_validators.append(LowerCaseLetterCountValidator(settings.PASSWORD_MIN_LOWERCASE_LETTERS))
        super(CustomPasswordPoliciesField, self).__init__(*args, **kwargs)



""" Url router for the main application
"""
from django.conf.urls import url, include

from core_main_app.views.user import views as user_views, ajax as user_ajax
from core_main_app.views.common import ajax as common_ajax, views as common_views
from core_main_app.utils.rendering import render

from rest_framework_swagger.views import get_swagger_view
schema_view = get_swagger_view(title="REST API")

urlpatterns = [
    url(r'^$', user_views.homepage, name='core_main_app_homepage'),

    url(r'^login', user_views.custom_login, name='core_main_app_login'),
    url(r'^logout', user_views.custom_logout, name='core_main_app_logout'),

    url(r'^rest/', include('core_main_app.rest.urls')),
    url(r'^data',  common_views.ViewData.as_view(), name='core_main_app_data_detail'),

    url(r'^template/versions/(?P<version_manager_id>\w+)', user_views.manage_template_versions,
        name='core_main_app_manage_template_versions'),
    url(r'^template/(?P<pk>[\w-]+)/edit/$', common_ajax.EditTemplateVersionManagerView.as_view(),
        name='core_main_app_edit_template'),
    url(r'^template/disable', common_ajax.disable_version_manager,
        name='core_main_app_disable_template'),
    url(r'^template/restore', common_ajax.restore_version_manager,
        name='core_main_app_restore_template'),

    url(r'^template/version/disable', common_ajax.disable_template_version_from_version_manager,
        name='core_main_app_disable_template_version'),
    url(r'^template/version/restore', common_ajax.restore_template_version_from_version_manager,
        name='core_main_app_restore_template_version'),
    url(r'^template/version/current', common_ajax.set_current_template_version_from_version_manager,
        name='core_main_app_set_current_template_version'),

    url(r'^xslt$', common_views.XSLTView.as_view(), name='core_main_app_xslt'),
    url(r'^xslt/upload$', common_views.UploadXSLTView.as_view(), name='core_main_app_upload_xslt'),
    url(r'^template/xslt/(?P<template_id>\w+)',
        common_views.TemplateXSLRenderingView.as_view(
            rendering=render,
            template_name="core_main_app/common/templates_xslt/main.html",
            save_redirect="core_main_app_manage_template_versions"
        ),
        name='core_main_app_template_xslt'),

    url(r'^edit-rights/(?P<workspace_id>\w+)$', common_views.EditWorkspaceRights.as_view(),
        name='core_main_edit_rights_workspace'),

    url(r'^create-workspace', user_ajax.create_workspace, name='core_main_create_workspace'),
    url(r'^change-workspace', user_ajax.load_form_change_workspace, name='core_main_change_workspace'),
    url(r'^assign-workspace', user_ajax.assign_workspace, name='core_main_assign_workspace'),
    url(r'^public-workspace', user_ajax.set_public_workspace, name='core_main_public_workspace'),

    url(r'^add-user-form', user_ajax.load_add_user_form, name='core_main_edit_rights_users_form'),
    url(r'^add-user-right-to-workspace', user_ajax.add_user_right_to_workspace,
        name='core_main_add_user_right_to_workspace'),
    url(r'^switch-right', user_ajax.switch_right, name='core_main_switch_right'),
    url(r'^remove-rights', user_ajax.remove_user_or_group_rights, name='core_main_remove_rights'),
    url(r'^add-group-form', user_ajax.load_add_group_form, name='core_main_edit_rights_groups_form'),
    url(r'^add-group-right-to-workspace', user_ajax.add_group_right_to_workspace,
        name='core_main_add_group_right_to_workspace'),
    url(r'^docs/api$', schema_view, name='swagger_view'),
]



""" Fixtures files for Workspace
"""

from core_main_app.utils.integration_tests.fixture_interface import FixtureInterface
from core_main_app.components.workspace import api as workspace_api


class WorkspaceFixtures(FixtureInterface):
    """ Workspace Fixture
    """
    def insert_data(self):
        pass

    @staticmethod
    def create_workspace(owner_id, title):
        return workspace_api.create_and_save(owner_id, title)



"""
    Xml operation test class
"""
import core_main_app.commons.exceptions as exceptions
from unittest import TestCase
from core_main_app.utils.xml import raw_xml_to_dict, remove_lists_from_xml_dict
from collections import OrderedDict


class TestRawToDict(TestCase):
    def test_raw_to_dict_valid(self):
        # Arrange
        raw_xml = '<root><test>Hello</test></root>'
        expected_dict = OrderedDict([(u'root', OrderedDict([(u'test', u'Hello')]))])

        # Act
        xml_dict = raw_xml_to_dict(raw_xml)

        # Assert
        self.assertEquals(expected_dict, xml_dict)

    def test_raw_to_dict_throws_exception_when_invalid_xml(self):
        # Arrange
        raw_xml = '<root><test>Hello</test?</root>'

        # Act # Assert
        with self.assertRaises(exceptions.XMLError):
            raw_xml_to_dict(raw_xml)


class TestRemoveListsFromXmlDict(TestCase):
    def test_remove_lists_from_xml_dict_empty_dict_does_nothing(self):
        # Arrange
        xml_dict = {}
        # Act
        remove_lists_from_xml_dict(xml_dict)
        # Assert
        self.assertTrue(xml_dict == {})

    def test_remove_lists_from_xml_dict_root_list_is_removed(self):
        # Arrange
        xml_dict = {
            'list': [{'value': 1},
                     {'value': 2}]
        }
        # Act
        remove_lists_from_xml_dict(xml_dict)
        # Assert
        self.assertTrue(xml_dict == {})

    def test_remove_lists_from_xml_dict_root_list_is_not_removed_if_smaller_than_max_size(self):
        # Arrange
        xml_dict = {
            'list': [{'value': 1},
                     {'value': 2}]
        }
        # Act
        remove_lists_from_xml_dict(xml_dict, 3)
        # Assert
        self.assertTrue(xml_dict == {
            'list': [{'value': 1},
                     {'value': 2}]
        })

    def test_remove_lists_from_xml_dict_sub_element_list_is_removed(self):
        # Arrange
        xml_dict = {
            'root': {
                'list': [{'value': 1}, {'value': 2}]
            }
        }
        # Act
        remove_lists_from_xml_dict(xml_dict)
        # Assert
        self.assertTrue(xml_dict == {'root': {}})

    def test_remove_lists_from_xml_dict_sub_element_list_is_not_removed_if_smaller_than_max_size(self):
        # Arrange
        xml_dict = {
            'root': {
                'list': [{'value': 1}, {'value': 2}]
            }
        }
        # Act
        remove_lists_from_xml_dict(xml_dict, 3)
        # Assert
        self.assertTrue(xml_dict == {
            'root': {
                'list': [{'value': 1}, {'value': 2}]
            }
        })

    def test_remove_lists_from_xml_dict_sub_elements_only_list_is_removed(self):
        # Arrange
        xml_dict = {
            'root': {
                'list': [{'value': 1}, {'value': 2}],
                'int': 3,
                'str': 'test',
                'dict': {
                    'value': 'test'
                }
            }
        }
        # Act
        remove_lists_from_xml_dict(xml_dict)
        # Assert
        self.assertTrue(xml_dict == {
            'root': {
                'int': 3,
                'str': 'test',
                'dict': {
                    'value': 'test'
                }
            }
        })




"""
    Workspace access control
"""

from core_main_app.utils.access_control.exceptions import AccessControlError


def is_workspace_owner_to_perform_action_for_others(func, workspace, new_user_id, user):
    """ Check if the user is the owner of the workspace to perform action for other user.

    Args:
        func
        workspace
        new_user_id
        user

    Returns:

    """
    if user.is_superuser:
        return func(workspace, new_user_id, user)

    _check_is_owner_workspace(workspace,  user)
    return func(workspace, new_user_id, user)


def is_workspace_owner(func, workspace, user):
    """ Check if the user is the owner of the workspace.

    Args:
        func:
        workspace:
        user:

    Returns:

    """
    if user.is_superuser:
        return func(workspace, user)

    _check_is_owner_workspace(workspace, user)
    return func(workspace, user)


def can_delete_workspace(func, workspace, user):
    """ Can user delete a workspace.

    Args:
        func:
        workspace:
        user:

    Returns:

    """
    if user.is_superuser:
        return func(workspace, user)

    _check_is_owner_workspace(workspace, user)
    return func(workspace, user)


def _check_is_owner_workspace(workspace, user):
    """ Check that user is the owner of the workspace.

    Args:
        workspace:
        user:

    Returns:

    """
    if workspace.owner != str(user.id):
        raise AccessControlError("The user does not have the permission. The user is not the owner of this workspace.")



""" TemplateXslRendering model
"""
from django_mongoengine import fields, Document
from mongoengine import errors as mongoengine_errors
from core_main_app.commons import exceptions
from core_main_app.components.template.models import Template
from core_main_app.components.xsl_transformation.models import XslTransformation
from mongoengine.queryset.base import CASCADE


class TemplateXslRendering(Document):
    """ TemplateXslRendering object
    """
    template = fields.ReferenceField(Template, blank=False, reverse_delete_rule=CASCADE, unique=True)
    list_xslt = fields.ReferenceField(XslTransformation, blank=True)
    detail_xslt = fields.ReferenceField(XslTransformation, blank=True)

    @staticmethod
    def get_by_id(template_xsl_rendering_id):
        """ Get a TemplateXslRendering document by its id.

        Args:
            template_xsl_rendering_id: Id.

        Returns:
            TemplateXslRendering object.

        Raises:
            DoesNotExist: The TemplateXslRendering doesn't exist.
            ModelError: Internal error during the process.

        """
        try:
            return TemplateXslRendering.objects.get(pk=str(template_xsl_rendering_id))
        except mongoengine_errors.DoesNotExist as e:
            raise exceptions.DoesNotExist(e.message)
        except Exception as ex:
            raise exceptions.ModelError(ex.message)

    @staticmethod
    def get_by_template_id(template_id):
        """Get TemplateXslRendering by its template id.

        Args:
            template_id: Template id.

        Returns:
            The TemplateXslRendering instance.

        Raises:
            DoesNotExist: The TemplateXslRendering doesn't exist.
            ModelError: Internal error during the process.

        """
        try:
            return TemplateXslRendering.objects.get(template=template_id)
        except mongoengine_errors.DoesNotExist as e:
            raise exceptions.DoesNotExist(e.message)
        except Exception as e:
            raise exceptions.ModelError(e.message)

    @staticmethod
    def get_all():
        """Get all TemplateXslRendering.

        Returns:
            List of TemplateXslRendering.

        """
        return TemplateXslRendering.objects.all()



""" Form needed for the user part of everything
"""
from django import forms

from core_main_app.commons.exceptions import DoesNotExist
from core_main_app.components.workspace import api as workspace_api
from core_main_app.components.user import api as user_api


class LoginForm(forms.Form):
    """ Custom login form for the user
    """
    username = forms.CharField(label="Username", required=True)
    password = forms.CharField(label="Password", required=True, widget=forms.PasswordInput)

    next_page = forms.CharField(widget=forms.HiddenInput)


class GroupRightForm(forms.Form):
    """
    Form to select group to add rights.
    """
    groups = forms.MultipleChoiceField(label='', required=True,
                                       widget=forms.SelectMultiple())
    GROUPS_OPTIONS = []

    def __init__(self, groups_with_no_access):
        self.GROUPS_OPTIONS = []

        # We sort by name, case sensitive
        sort_groups = sorted(groups_with_no_access, key=lambda s: s.name.lower())

        # We add them
        for group in sort_groups:
            self.GROUPS_OPTIONS.append((group.id, group.name))

        super(GroupRightForm, self).__init__()
        self.fields['groups'].choices = []
        self.fields['groups'].choices = self.GROUPS_OPTIONS


class UserRightForm(forms.Form):
    """
    Form to select user to add rights.
    """
    users = forms.MultipleChoiceField(label='', required=True, widget=forms.SelectMultiple())
    USERS_OPTIONS = []

    def __init__(self, users_with_no_access):
        self.USERS_OPTIONS = []

        # We sort by username, case sensitive
        sort_users = sorted(users_with_no_access, key=lambda s: s.username.lower())

        # We add them
        for user in sort_users:
            self.USERS_OPTIONS.append((user.id, user.username))

        super(UserRightForm, self).__init__()
        self.fields['users'].choices = []
        self.fields['users'].choices = self.USERS_OPTIONS


class WorkspaceForm(forms.Form):
    """
    Form to create the workspace.
    """
    workspace_name = forms.CharField( max_length=100)


class ChangeWorkspaceForm(forms.Form):
    """
    Form to select a workspace.
    """
    workspaces = forms.ChoiceField(label='', required=True, widget=forms.Select(attrs={"class": "form-control"}))
    WORKSPACES_OPTIONS = []

    def __init__(self, user, list_current_workspace=[], is_administration=False):
        self.WORKSPACES_OPTIONS = []
        self.WORKSPACES_OPTIONS.append(('', '-----------'))

        # We retrieve all workspaces with write access, or all workspaces if administration
        if is_administration:
            all_workspaces = workspace_api.get_all()
        else:
            all_workspaces = workspace_api.get_all_workspaces_with_write_access_by_user(user)

        if len(all_workspaces) == 0:
            raise DoesNotExist("You don't have access to any workspaces with sufficient rights to assign a document.")

        # We sort by title, case insensitive
        sort_workspaces = sorted(all_workspaces, key=lambda s: s.title.lower())

        # We add them
        for workspace in sort_workspaces:
            if list_current_workspace == [] or\
                    (len(list_current_workspace) > 0 and workspace not in list_current_workspace):
                self.WORKSPACES_OPTIONS.append((workspace.id,
                                                workspace.title + " (" + user_api.get_user_by_id(workspace.owner).username + ")"))

        super(ChangeWorkspaceForm, self).__init__()
        self.fields['workspaces'].choices = []
        self.fields['workspaces'].choices = self.WORKSPACES_OPTIONS



"""
Version Manager API
"""

from core_main_app.commons import exceptions
from core_main_app.commons.exceptions import ApiError
from core_main_app.components.version_manager.models import VersionManager


def get(version_manager_id):
    """Get a version manager by its id.

    Args:
        version_manager_id:

    Returns:

    """
    return VersionManager.get_by_id(version_manager_id)


def get_by_id_list(list_id):
    """Get a version managers with the given id list.

    Args:
        list_id:

    Returns:

    """
    return VersionManager.get_by_id_list(list_id)


def get_from_version(version):
    """Return a version manager from a version.

    Args:
        version:

    Returns:

    """
    version_managers = VersionManager.get_all()
    for version_manager in version_managers:
        if str(version.id) in version_manager.versions:
            return version_manager
    raise exceptions.ApiError("No version manager could be found for this version.")


def disable(version_manager):
    """Disable an object, then saves it.

    Args:
        version_manager:

    Returns:

    """
    version_manager.disable()
    return upsert(version_manager)


def restore(version_manager):
    """Restore an object, then saves it.

    Args:
        version_manager:

    Returns:

    """
    version_manager.restore()
    return upsert(version_manager)


def restore_version(version):
    """Disable a version of the object, then saves it.

    Args:
        version:

    Returns:

    """
    version_manager = get_from_version(version)
    try:
        version_manager.restore_version(version)
    except ValueError as value_error:
        raise ApiError('Unable to restore this version: status is not disabled.')
    return upsert(version_manager)


def disable_version(version, new_current=None):
    """Disable a version of the object, then saves it.

    Args:
        version:
        new_current:

    Returns:

    """
    version_manager = get_from_version(version)

    # try to delete the current version
    if version_manager.current == str(version.id):
        # no version to be current provided
        if new_current is None:
            raise exceptions.ApiError('Unable to disable the current version.')

        # id doesn't match a version
        if new_current.id not in version_manager.versions:
            raise exceptions.ApiError('The id provided to be the next current version, could not be found.')

        # set the new current version
        version_manager.set_current_version(new_current)

    # disable the version
    version_manager.disable_version(version)
    return upsert(version_manager)


def set_current(version):
    """Set the current version of the object, then saves it.

    Args:
        version:

    Returns:

    """
    version_manager = get_from_version(version)

    # a disabled version cannot be current
    if str(version.id) in version_manager.get_disabled_versions():
        raise exceptions.ApiError("Unable to set the current version because it is disabled.")

    version_manager.set_current_version(version)
    return upsert(version_manager)


def get_current(version_manager):
    """Get the current version of the version manager.

    Args:
        version_manager:

    Returns:

    """
    return version_manager.current


def upsert(version_manager):
    """Save or update version manager.

    Args:
        version_manager:

    Returns:

    """
    return version_manager.save_version_manager()


def insert_version(version_manager, version):
    """Insert a version in the version manager, then saves it.

    Args:
        version_manager:
        version:

    Returns:

    """
    version_manager.insert(version)

    # first version inserted, set it as current
    if len(version_manager.versions) == 1:
        version_manager.set_current_version(version)

    return upsert(version_manager)


def get_global_version_managers():
    """Return all Version Managers with user set to None.

    Returns:

    """
    return VersionManager.get_global_version_managers()


def get_active_global_version_manager_by_title(version_manager_title):
    """Return active Version Manager by its title with user set to None.

    Args:
        version_manager_title: Version Manager title

    Returns:
        Version Manager instance

    """
    return VersionManager.get_active_global_version_manager_by_title(version_manager_title)


def get_version_number(version_manager, version):
    """Return version number from version id.

    Args:
        version_manager:
        version:

    Returns:

    """
    return version_manager.get_version_number(version)


def get_version_by_number(version_manager, version_number):
    """Return the version by its version number.

    Args:
        version_manager:
        version_number: Number of the version.

    Returns:

    """
    return version_manager.get_version_by_number(version_number)



"""Url router for the REST API
"""
from django.conf.urls import url

from rest_framework.urlpatterns import format_suffix_patterns

import core_main_app.rest.xsl_transformation.views as xslTransformationList_view
from core_main_app.rest.blob import views as blob_views
from core_main_app.rest.data import views as data_views
from core_main_app.rest.template import views as template_views
from core_main_app.rest.template_version_manager import views as template_version_manager_views
from core_main_app.rest.workspace import views as workspace_views

urlpatterns = [
    url(r'^template-version-manager/global/$',
        template_version_manager_views.GlobalTemplateVersionManagerList.as_view(),
        name='core_main_app_rest_template_version_manager_global_list'),

    url(r'^template-version-manager/user/$',
        template_version_manager_views.UserTemplateVersionManagerList.as_view(),
        name='core_main_app_rest_template_version_manager_user_list'),

    url(r'^template-version-manager/(?P<pk>\w+)/$',
        template_version_manager_views.TemplateVersionManagerDetail.as_view(),
        name='core_main_app_rest_template_version_manager_detail'),

    url(r'^template-version-manager/(?P<pk>\w+)/version/$',
        template_version_manager_views.TemplateVersion.as_view(),
        name='core_main_app_rest_template_version'),

    url(r'^template-version-manager/(?P<pk>\w+)/disable/$',
        template_version_manager_views.DisableTemplateVersionManager.as_view(),
        name='core_main_app_rest_template_version_manager_disable'),

    url(r'^template-version-manager/(?P<pk>\w+)/restore/$',
        template_version_manager_views.RestoreTemplateVersionManager.as_view(),
        name='core_main_app_rest_template_version_manager_restore'),

    url(r'^template/version/(?P<pk>\w+)/current/$',
        template_version_manager_views.CurrentTemplateVersion.as_view(),
        name='core_main_app_rest_template_version_current'),

    url(r'^template/version/(?P<pk>\w+)/disable/$',
        template_version_manager_views.DisableTemplateVersion.as_view(),
        name='core_main_app_rest_template_version_disable'),

    url(r'^template/version/(?P<pk>\w+)/restore/$',
        template_version_manager_views.RestoreTemplateVersion.as_view(),
        name='core_main_app_rest_template_version_restore'),

    url(r'^template/global/$',
        template_version_manager_views.GlobalTemplateList.as_view(),
        name='core_main_app_rest_global_template_list'),

    url(r'^template/user/$',
        template_version_manager_views.UserTemplateList.as_view(),
        name='core_main_app_rest_user_template_list'),

    url(r'^template/(?P<pk>\w+)/download/$', template_views.TemplateDownload.as_view(),
        name='core_main_app_rest_template_download'),

    url(r'^template/(?P<pk>\w+)/$', template_views.TemplateDetail.as_view(),
        name='core_main_app_rest_template_detail'),

    url(r'^data/$', data_views.DataList.as_view(),
        name='core_main_app_rest_data_list'),

    url(r'^data/download/(?P<pk>\w+)/$', data_views.DataDownload.as_view(),
        name='core_main_app_rest_data_download'),

    url(r'^data/get-full$', data_views.get_by_id_with_template_info,
        name='core_main_app_rest_data_get_by_id_with_template_info'),

    url(r'^data/query/keyword/$', data_views.ExecuteLocalKeywordQueryView.as_view(),
        name='core_explore_common_local_query_keyword'),

    url(r'^data/query/$', data_views.ExecuteLocalQueryView.as_view(),
        name='core_explore_common_local_query'),

    url(r'^data/(?P<pk>\w+)/$', data_views.DataDetail.as_view(),
        name='core_main_app_rest_data_detail'),

    url(r'^blob/$', blob_views.BlobList.as_view(),
        name='core_main_app_rest_blob_list'),

    url(r'^blobs/delete/$', blob_views.BlobDeleteList.as_view(),
        name='core_main_app_rest_blob_delete_list'),

    url(r'^blob/(?P<pk>\w+)/$', blob_views.BlobDetail.as_view(),
        name='core_main_app_rest_blob_detail'),

    url(r'^blob/download/(?P<pk>\w+)/$', blob_views.BlobDownload.as_view(),
        name='core_main_app_rest_blob_download'),

    url(r'^xslt/$', xslTransformationList_view.XslTransformationList.as_view(),
        name='core_main_app_rest_xslt'),

    url(r'^xslt/transform/$', xslTransformationList_view.XslTransformationTransform.as_view(),
        name='core_main_app_rest_xslt_transform'),

    url(r'^xslt/(?P<pk>\w+)/$', xslTransformationList_view.XslTransformationDetail.as_view(),
        name='core_main_app_rest_xslt_detail'),

    url(r'^workspace/$', workspace_views.WorkspaceList.as_view(),
        name='core_main_app_rest_workspace_list'),

    url(r'^workspace/(?P<pk>\w+)/detail/$', workspace_views.WorkspaceDetail.as_view(),
        name='core_main_app_rest_workspace_detail'),

    url(r'^workspace/read_access', workspace_views.get_workspaces_with_read_access,
        name='core_main_app_rest_workspace_get_all_workspaces_with_read_access_by_user'),

    url(r'^workspace/write_access', workspace_views.get_workspaces_with_write_access,
        name='core_main_app_rest_workspace_get_all_workspaces_with_write_access_by_user'),

    url(r'^workspace/(?P<pk>\w+)/is_public/$', workspace_views.is_workspace_public,
        name='core_main_app_rest_workspace_is_public'),

    url(r'^workspace/(?P<pk>\w+)/set_public/$', workspace_views.set_workspace_public,
        name='core_main_app_rest_workspace_set_public'),

    url(r'^workspace/(?P<pk>\w+)/list_user_can_read/$', workspace_views.get_list_user_can_read_workspace,
        name='core_main_app_rest_workspace_list_user_can_read'),

    url(r'^workspace/(?P<pk>\w+)/list_user_can_write/$', workspace_views.get_list_user_can_write_workspace,
        name='core_main_app_rest_workspace_list_user_can_write'),

    url(r'^workspace/(?P<pk>\w+)/list_user_can_access/$', workspace_views.get_list_user_can_access_workspace,
        name='core_main_app_rest_workspace_list_user_can_access'),

    url(r'^workspace/(?P<pk>\w+)/list_group_can_read/$', workspace_views.get_list_group_can_read_workspace,
        name='core_main_app_rest_workspace_list_group_can_read'),

    url(r'^workspace/(?P<pk>\w+)/list_group_can_write/$', workspace_views.get_list_group_can_write_workspace,
        name='core_main_app_rest_workspace_list_group_can_write'),

    url(r'^workspace/(?P<pk>\w+)/list_group_can_access/$', workspace_views.get_list_group_can_access_workspace,
        name='core_main_app_rest_workspace_list_group_can_access'),

    url(r'^workspace/(?P<pk>\w+)/add_read_right_to_user/(?P<user_id>\w+)/$',
        workspace_views.add_user_read_right_to_workspace,
        name='core_main_app_rest_workspace_add_user_read'),

    url(r'^workspace/(?P<pk>\w+)/add_write_right_to_user/(?P<user_id>\w+)/$',
        workspace_views.add_user_write_right_to_workspace,
        name='core_main_app_rest_workspace_add_user_write'),

    url(r'^workspace/(?P<pk>\w+)/add_read_right_to_group/(?P<group_id>\w+)/$',
        workspace_views.add_group_read_right_to_workspace,
        name='core_main_app_rest_workspace_add_group_read'),

    url(r'^workspace/(?P<pk>\w+)/add_write_right_to_group/(?P<group_id>\w+)/$',
        workspace_views.add_group_write_right_to_workspace,
        name='core_main_app_rest_workspace_add_group_write'),

    url(r'^workspace/(?P<pk>\w+)/remove_read_right_to_user/(?P<user_id>\w+)/$',
        workspace_views.remove_user_read_right_to_workspace,
        name='core_main_app_rest_workspace_remove_user_read'),

    url(r'^workspace/(?P<pk>\w+)/remove_write_right_to_user/(?P<user_id>\w+)/$',
        workspace_views.remove_user_write_right_to_workspace,
        name='core_main_app_rest_workspace_remove_user_write'),

    url(r'^workspace/(?P<pk>\w+)/remove_read_right_to_group/(?P<group_id>\w+)/$',
        workspace_views.remove_group_read_right_to_workspace,
        name='core_main_app_rest_workspace_remove_group_read'),

    url(r'^workspace/(?P<pk>\w+)/remove_write_right_to_group/(?P<group_id>\w+)/$',
        workspace_views.remove_group_write_right_to_workspace,
        name='core_main_app_rest_workspace_remove_group_write'),
]

urlpatterns = format_suffix_patterns(urlpatterns)
